/*
**  glcurrent.c
**
** Written by Ian Buck
** 6/15/2000
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include <stdlib.h>
#include <stdio.h>
#include "glcontext.h"
#include "glcurrent.h"
#include "glhw.h"
#include "glerror.h"


void 
__glcurrent_initbits (GLcurrentbits *, GLconfig *) {
	/* do nothing */
}

void
__glcurrent_switch (GLcurrentbits *c, GLbitvalue bitID,
					 GLcurrentstate *current, GLcurrentstate *target) {
	GLbitvalue nbitID = ~bitID;

	if (c->enable & bitID) {
		if (current->normalize != target->normalize) {
			if (target->normalize == GL_TRUE)
				__glhw_Enable(GL_NORMALIZE);
			else
				__glhw_Disable(GL_NORMALIZE);
			c->enable = GLBITS_ONES;
		}
		c->enable &= nbitID;
		c->dirty = GLBITS_ONES;
	}

	if (c->color & bitID) {
		switch (target->colortype) {
		case GL_BYTE:
			__glhw_Color4bv ((GLbyte *) &(target->colorb));
			break;
		case GL_UNSIGNED_BYTE:
			__glhw_Color4ubv ((GLubyte *) &(target->colorub));
			break;
		case GL_SHORT:
			__glhw_Color4sv ((GLshort *) &(target->colors));
			break;
		case GL_UNSIGNED_SHORT:
			__glhw_Color4usv ((GLushort *) &(target->colorus));
			break;
		case GL_INT:
			__glhw_Color4iv ((GLint *) &(target->colori));
			break;
		case GL_UNSIGNED_INT:
			__glhw_Color4uiv ((GLuint *) &(target->colorui));
			break;
		case GL_FLOAT:
			__glhw_Color4fv ((GLfloat *) &(target->colorf));
			break;
		case GL_DOUBLE:
			__glhw_Color4dv ((GLdouble *) &(target->colord));
			break;
		default:
			UNIMPLEMENTED();
		}
		/* 
		** Currently no value check, so set all invalid
		*/
		c->color = nbitID;
		c->dirty = GLBITS_ONES;
	}

	if (c->raster & bitID) {
		if (target->rastervalid) {
			if (target->rasterpos_pre.x != current->rasterpos.x ||
				target->rasterpos_pre.y != current->rasterpos.y) {
					GLvectorf p;
					p.x = target->rasterpos_pre.x - current->rasterpos.x;
					p.y = target->rasterpos_pre.y - current->rasterpos.y;
					__glhw_Bitmap(0, 0, 0.0f, 0.0f, p.x, p.y, 0);
					c->raster = GLBITS_ONES;
					c->dirty  = GLBITS_ONES;
			}
		}
		c->raster &= nbitID;
	}

	c->dirty &= nbitID;
}
void
__glcurrent_sync (GLcurrentbits *c, GLbitvalue bitID,
					 GLcurrentstate *current, GLcurrentstate *target) {
	GLbitvalue nbitID = ~bitID;

	if (c->enable & bitID) {
		if (current->normalize != target->normalize) {
			if (target->normalize == GL_TRUE)
				__glhw_Enable(GL_NORMALIZE);
			else
				__glhw_Disable(GL_NORMALIZE);
			current->normalize = target->normalize;
		}
		c->enable &= nbitID;
	}

	if (c->color & bitID) {
		switch (target->colortype) {
		case GL_BYTE:
			__glhw_Color4bv ((GLbyte *) &(target->colorb));
			current->colorb = target->colorb;
			current->colortype = GL_BYTE;
			break;
		case GL_UNSIGNED_BYTE:
			__glhw_Color4ubv ((GLubyte *) &(target->colorub));
			current->colorub = target->colorub;
			current->colortype = GL_UNSIGNED_BYTE;
			break;
		case GL_SHORT:
			__glhw_Color4sv ((GLshort *) &(target->colors));
			current->colors = target->colors;
			current->colortype = GL_SHORT;
			break;
		case GL_UNSIGNED_SHORT:
			__glhw_Color4usv ((GLushort *) &(target->colorus));
			current->colorus = target->colorus;
			current->colortype = GL_UNSIGNED_SHORT;
			break;
		case GL_INT:
			__glhw_Color4iv ((GLint *) &(target->colori));
			current->colori = target->colori;
			current->colortype = GL_INT;
			break;
		case GL_UNSIGNED_INT:
			__glhw_Color4uiv ((GLuint *) &(target->colorui));
			current->colorui = target->colorui;
			current->colortype = GL_UNSIGNED_INT;
			break;
		case GL_FLOAT:
			__glhw_Color4fv ((GLfloat *) &(target->colorf));
			current->colorf = target->colorf;
			current->colortype = GL_FLOAT;
			break;
		case GL_DOUBLE:
			__glhw_Color4dv ((GLdouble *) &(target->colord));
			current->colord = target->colord;
			current->colortype = GL_DOUBLE;
			break;
		default:
			UNIMPLEMENTED();
		}
		c->color &= nbitID;
	}

	if (c->raster & bitID) {
		current->rastervalid = target->rastervalid;
		if (target->rastervalid) {
			if (target->rasterpos_pre.x != current->rasterpos.x ||
				target->rasterpos_pre.y != current->rasterpos.y) {
					GLvectorf p;
					p.x = target->rasterpos_pre.x - current->rasterpos.x;
					p.y = target->rasterpos_pre.y - current->rasterpos.y;
					__glhw_Bitmap(0, 0, 0.0f, 0.0f, p.x, p.y, 0);
			}
			current->rasterpos = target->rasterpos;
		}
		c->raster &= nbitID;
	}

	c->dirty &= nbitID;
}

void
__glcurrent_init (GLcurrentstate *c, GLconfig *cfg) {
	
	GLvectorf empty_vector;	
	GLvectorf zero_vector;
	GLvectorub one_vector;

	empty_vector.x = 0.0f;
	empty_vector.y = 0.0f;
	empty_vector.z = 0.0f;
	empty_vector.w = 0.0f;

	zero_vector.x = 0.0f;
	zero_vector.y = 0.0f;
	zero_vector.z = 0.0f;
	zero_vector.w = 1.0f;

	one_vector.x = 255;
	one_vector.y = 0;
	one_vector.z = 0;
	one_vector.w = 0;

	c->colorub = one_vector;
	c->colortype = GL_UNSIGNED_BYTE;

	c->indexf = 1.0f;
	c->indextype = GL_FLOAT;

	c->texcoordf = zero_vector;
	c->texcoordtype = GL_FLOAT;

	c->normalf = empty_vector;
	c->normalf.z = 1.0f;
	c->normaltype = GL_FLOAT;

    /* Compute the raster pos */
	if (cfg->id < cfg->numprojectors) {
		GLrecti *b;
		b = cfg->bounds + cfg->id;
		c->rasterpos.x = (GLfloat) b->x1;
		c->rasterpos.y = (GLfloat) b->y1;
	} else {
		c->rasterpos.x = 0.0f;
		c->rasterpos.y = 0.0f;
	}
	c->rasterpos.z = 0.0f;
	c->rasterpos.w = 1.0f;

	c->rasterpos_pre = c->rasterpos;

	c->rasterdistance = 0.0f;
	c->rastercolor.r = 1.0f;
	c->rastercolor.g = 1.0f;
	c->rastercolor.b = 1.0f;
	c->rastercolor.a = 1.0f;
	c->rastertexture.s = 0.0f;
	c->rastertexture.t = 0.0f;
	c->rastertexture.p = 0.0f;
	c->rastertexture.q = 1.0f;
	c->rastervalid = GL_TRUE;
	c->rasterindex = 1.0f;

	c->edgeflag = GL_TRUE;
	c->normalize = GL_FALSE;

	c->beginend = GL_FALSE;
	c->beginendnum = 0;
	c->mode = 0x10;

	c->beginend_orphan = GL_FALSE;
	c->beginendnum_orphan = 0;
	c->mode_orphan = 0x10;
}

void
__glcurrent_destroy (GLcurrentstate *) {
/*
** Do Nothing
*/
}

void GLSTATE_DECL
__glstate_Begin (GLenum mode) {
	GLcontext *g = GetCurrentContext();
	GLcurrentstate *c = &g->current;
	GLbucketstate *b = &g->bucket;

	if (mode > GL_POLYGON)
		if (__glerror (__LINE__, __FILE__, GL_INVALID_ENUM,
			"Begin called with invalid mode: %d", mode))
			return;

	if (c->beginend)
		if (__glerror (__LINE__, __FILE__, GL_INVALID_OPERATION,
			"Begin called inside begin/end"))
			return;

	if (*b->count > *b->max)
		b->flush(g);
	
	__glhw_Begin (mode);

	/* Begin the bucketing */
	if (!IsBucketingActive()) 
		__glbucket_begin();

	c->beginend = GL_TRUE;
	c->mode = mode;
	c->beginendnum++;
}

void GLSTATE_DECL
__glstate_End (void) {
	GLcontext *g = GetCurrentContext();
	GLcurrentstate *c = &g->current;

	if (!c->beginend)
		if (__glerror (__LINE__, __FILE__, GL_INVALID_OPERATION,
			"End called without preceeding Begin"))
			return;

	__glhw_End();

	c->beginend = GL_FALSE;

}

void GLSTATE_DECL
__gltrack_Begin (GLenum mode) {
	GLcontext *g = GetCurrentContext();
	GLcurrentstate *c = &g->current;

	__glhw_Begin (mode);

	c->beginend = GL_TRUE;
	c->mode = mode;
}

void GLSTATE_DECL
__gltrack_End (void) {
	GLcontext *g = GetCurrentContext();
	GLcurrentstate *c = &g->current;
	
	__glhw_End();

	c->beginend = GL_FALSE;
}
