/*
** config.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include <stdio.h>
#include <stdlib.h>

#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_client.h"
#include "wiregl/include/wiregl_common.h"
#include "wiregl/include/wiregl_barrier.h"
#include "wiregl/include/wiregl_instrument.h"

int wiregl_mural_width = 0, wiregl_mural_height = 0;
int swap_barrier = 0;
char *barrier_server;

int print_bytes_per_server = 0;
int print_bytes_per_frame_per_server = 0;
int print_min_bytes_sent = 0;
int print_frame_rate = 0;
int print_flush_time = 0;
int print_send_time = 0;
int print_diff_context_time = 0;
int print_total_bytes_sent = 0;
int print_total_time = 0;

WireGLPipeServer *cur_pipeserver = NULL;

void InitiateServerConnections( )
{
	WireGLClientConnection connection;
	for (int i = 0 ; i < num_pipe_servers; i++)
	{
		connection.window_x = pipe_servers[i].window_x;
		connection.window_y = pipe_servers[i].window_y;
		connection.window_width = pipe_servers[i].w;
		connection.window_height = pipe_servers[i].h;
		connection.feather_top_dice = pipe_servers[i].feather_top_dice;
		connection.feather_top_size = pipe_servers[i].feather_top_size;
		connection.feather_top[0] = pipe_servers[i].feather_top[0];
		connection.feather_top[1] = pipe_servers[i].feather_top[1];
		connection.feather_top[2] = pipe_servers[i].feather_top[2];
		connection.feather_top[3] = pipe_servers[i].feather_top[3];
		connection.feather_bottom_dice = pipe_servers[i].feather_bottom_dice;
		connection.feather_bottom_size = pipe_servers[i].feather_bottom_size;
		connection.feather_bottom[0] = pipe_servers[i].feather_bottom[0];
		connection.feather_bottom[1] = pipe_servers[i].feather_bottom[1];
		connection.feather_bottom[2] = pipe_servers[i].feather_bottom[2];
		connection.feather_bottom[3] = pipe_servers[i].feather_bottom[3];
		connection.feather_left_dice = pipe_servers[i].feather_left_dice;
		connection.feather_left_size = pipe_servers[i].feather_left_size;
		connection.feather_left[0] = pipe_servers[i].feather_left[0];
		connection.feather_left[1] = pipe_servers[i].feather_left[1];
		connection.feather_left[2] = pipe_servers[i].feather_left[2];
		connection.feather_left[3] = pipe_servers[i].feather_left[3];
		connection.feather_right_dice = pipe_servers[i].feather_right_dice;
		connection.feather_right_size = pipe_servers[i].feather_right_size;
		connection.feather_right[0] = pipe_servers[i].feather_right[0];
		connection.feather_right[1] = pipe_servers[i].feather_right[1];
		connection.feather_right[2] = pipe_servers[i].feather_right[2];
		connection.feather_right[3] = pipe_servers[i].feather_right[3];

		pipe_servers[i].connection = wireGLConnectToServer( pipe_servers[i].server, WIREGL_PORT, &connection );
		
		wireGLFinishConnectToServer( pipe_servers[i].connection );

		pipe_servers[i].connection->wireGLRegisterMemoryForDMA( pipe_servers[i].connection, pipe_servers[i].data_start, pipe_server_buffer_size );
		pipe_servers[i].connection->wireGLRegisterMemoryForDMA( pipe_servers[i].connection, pipe_servers[i].opcode_start , pipe_server_buffer_size );
	}
}

void ParseNumPipeservers( char *buf, void * )
{
	sscanf( buf, "%d", &(num_pipe_servers) );
	pipe_servers = (WireGLPipeServer *) wireGLAlloc( num_pipe_servers * sizeof( *pipe_servers ));
	for (int i = 0 ; i < num_pipe_servers; i++)
	{
		pipe_servers[i].data_start  = (unsigned char *) wireGLAlloc( pipe_server_buffer_size );
		pipe_servers[i].opcode_start = (unsigned char *) wireGLAlloc( pipe_server_buffer_size );

		pipe_servers[i].data_current = pipe_servers[i].data_start;
		pipe_servers[i].opcode_current = pipe_servers[i].opcode_start;

		pipe_servers[i].data_end = pipe_servers[i].data_start + pipe_server_buffer_size;
		pipe_servers[i].index = i;
		pipe_servers[i].w = 1024;
		pipe_servers[i].h = 768;
		pipe_servers[i].mural_x = 0;
		pipe_servers[i].mural_y = 0;
		pipe_servers[i].window_x = 0;
		pipe_servers[i].window_y = 0;
		pipe_servers[i].feather_top_dice = 0;
		pipe_servers[i].feather_top_size = 0;
		pipe_servers[i].feather_top[0] = 0.0f;
		pipe_servers[i].feather_top[1] = 0.0f;
		pipe_servers[i].feather_top[2] = 0.0f;
		pipe_servers[i].feather_top[3] = 0.0f;
		pipe_servers[i].feather_bottom_dice = 0;
		pipe_servers[i].feather_bottom_size = 0;
		pipe_servers[i].feather_bottom[0] = 0.0f;
		pipe_servers[i].feather_bottom[1] = 0.0f;
		pipe_servers[i].feather_bottom[2] = 0.0f;
		pipe_servers[i].feather_bottom[3] = 0.0f;
		pipe_servers[i].feather_left_dice = 0;
		pipe_servers[i].feather_left_size = 0;
		pipe_servers[i].feather_left[0] = 0.0f;
		pipe_servers[i].feather_left[1] = 0.0f;
		pipe_servers[i].feather_left[2] = 0.0f;
		pipe_servers[i].feather_left[3] = 0.0f;
		pipe_servers[i].feather_right_dice = 0;
		pipe_servers[i].feather_right_size = 0;
		pipe_servers[i].feather_right[0] = 0.0f;
		pipe_servers[i].feather_right[1] = 0.0f;
		pipe_servers[i].feather_right[2] = 0.0f;
		pipe_servers[i].feather_right[3] = 0.0f;
	}
}

void ParseSetPipeserver( char *buf, void * )
{
	if (!num_pipe_servers)
		wireGLSimpleError( "I don't know how many pipeservers there are yet!" );
	
	int p;
	sscanf( buf, "%d", &p );
	if (p < 1 || p > num_pipe_servers)
		wireGLSimpleError( "Unknown pipeserver: %d (they start from one)", p );
	cur_pipeserver = pipe_servers + p - 1;
}

void ParseServer( char *buf, void * )
{
	cur_pipeserver->server = strdup( buf );
}

void ParseMuralExtent( char *buf, void * )
{
	sscanf( buf, "%d %d %d %d", &(cur_pipeserver->mural_x), &(cur_pipeserver->mural_y),
															&(cur_pipeserver->w),       &(cur_pipeserver->h) );
	if (cur_pipeserver->mural_x + cur_pipeserver->w > wiregl_mural_width)
		wiregl_mural_width = cur_pipeserver->mural_x + cur_pipeserver->w;
	if (cur_pipeserver->mural_y + cur_pipeserver->h > wiregl_mural_height)
		wiregl_mural_height = cur_pipeserver->mural_y + cur_pipeserver->h;
}

void ParseSwapBarrier( char *buf, void * )
{
/*
	if ( num_pipe_servers > 1 )
	{
		swap_barrier = 1;
		WireGLConnection *bconn = wireGLBarrierConnect( buf );
		wireGLBarrierCreate( bconn, PIPESERVER_BARRIER, num_pipe_servers );
		barrier_server = strdup( buf );
	}
*/
}

void ParsePrintStats( char *buf, void * )
{
	char name[1024];
	int i;

	if (!strcmp( buf, "bytes_per_server" ))
	{
		print_bytes_per_server = 1;
		wireGLInstrumentRegisterPerServerCounter( WIREGL_BYTES_PER_SERVER, num_pipe_servers, "Bytes per server" );
	}
	else if (!strcmp( buf, "bytes_per_frame_per_server" ) )
	{
		print_bytes_per_frame_per_server = 1;
		for (i = 0 ; i < num_pipe_servers ; i++)
		{
			sprintf( name, "Bytes per frame[%d]", i+1 );
			wireGLInstrumentRegisterPerFrameCounter( WIREGL_BYTES_PER_SERVER_PER_FRAME + i, name );
		}
	}
	else if (!strcmp( buf, "min_bytes_sent" ) )
	{
		print_min_bytes_sent = 1;
		wireGLInstrumentRegisterCounter( WIREGL_MIN_BYTES_SENT, "Minimum bytes to be sent" );
	}
	else if (!strcmp( buf, "frame_rate" ) )
	{
		print_frame_rate = 1;
		wireGLInstrumentRegisterTimer( WIREGL_FRAME_TIMER, "Frame time" );
		wireGLInstrumentRegisterPerFrameCounter( WIREGL_FRAME_TIMER, "Frame rate" );
		wireGLInstrumentStartTimer( WIREGL_FRAME_TIMER );
	}
	else if (!strcmp( buf, "flush_time" ) )
	{
		print_flush_time = 1;
		wireGLInstrumentRegisterTimer( WIREGL_FLUSH_TIMER, "Flush time" );
		wireGLInstrumentRegisterPerFrameCounter( WIREGL_FLUSH_TIMER, "Flush time" );
	}
	else if (!strcmp( buf, "send_time" ) )
	{
		print_send_time = 1;
		wireGLInstrumentRegisterTimer( WIREGL_SEND_TIMER, "Send time" );
		wireGLInstrumentRegisterPerFrameCounter( WIREGL_SEND_TIMER, "Send time" );
	}
	else if (!strcmp( buf, "diff_context_time" ) )
	{
		print_diff_context_time = 1;
		wireGLInstrumentRegisterTimer( WIREGL_DIFF_CONTEXT_TIMER, "Diff context time" );
		wireGLInstrumentRegisterPerFrameCounter( WIREGL_DIFF_CONTEXT_TIMER, "Diff context time" );
	}
	else if (!strcmp( buf, "bytes_sent" ) )
	{
		print_total_bytes_sent = 1;
		wireGLInstrumentRegisterCounter( WIREGL_TOTAL_BYTES_SENT, "Total bytes sent" );
	}
	else if (!strcmp( buf, "total_time" ) )
	{
		print_total_time = 1;
		wireGLInstrumentRegisterTimer( WIREGL_TOTAL_TIME, "Total time" );
		wireGLInstrumentStartTimer( WIREGL_TOTAL_TIME );
	}
}

void ParseStatsFile( char *buf, void * ) 
{
	wireGLInstrumentToFile( buf );
}

void ParseMuralWindow( char *buf, void * )
{
	sscanf( buf, "%d %d", &(cur_pipeserver->window_x), &(cur_pipeserver->window_y) );
}

void __ParseFeather( char *buf, int *size, float cp[])
{
	sscanf( buf, "%d %f %f %f %f", size, cp, cp+1, cp+2, cp+3);
}
		
void ParseFeatherTop( char *buf, void * ) {
	__ParseFeather( buf, &cur_pipeserver->feather_top_size, 
		cur_pipeserver->feather_top);
}

void ParseFeatherBottom( char *buf, void * ) {
	__ParseFeather( buf, &cur_pipeserver->feather_bottom_size,
		cur_pipeserver->feather_bottom);
}

void ParseFeatherLeft( char *buf, void * ) {
	__ParseFeather( buf, &cur_pipeserver->feather_left_size,
		cur_pipeserver->feather_left);
}

void ParseFeatherRight( char *buf, void * ) {
	__ParseFeather( buf, &cur_pipeserver->feather_right_size,
		cur_pipeserver->feather_right);
}

static WireGLParseStruct wiregl_parse[] = {
	{ "num_pipeservers", ParseNumPipeservers },
	{ "set_pipeserver", ParseSetPipeserver },
	{ "server", ParseServer },
	{ "mural_extent", ParseMuralExtent },
	{ "swap_barrier", ParseSwapBarrier },
	{ "print_stats", ParsePrintStats },
	{ "stats_file", ParseStatsFile },
	{ "mural_window", ParseMuralWindow },
	{ "feather_top", ParseFeatherTop },
	{ "feather_bottom", ParseFeatherBottom },
	{ "feather_left", ParseFeatherLeft },
	{ "feather_right", ParseFeatherRight },
	{ NULL, NULL }
};

void wireGLClientParseConfig( char *filename )
{
	FILE *fp = NULL;
	char fname[1024];

	char *wiregl_path, *temp;

	int found_it = 0;

	wiregl_path = getenv("WIREGL_PATH");
	if (!wiregl_path)
	{
		wiregl_path = strdup( "." );
	}

	temp = wiregl_path; 
	while (!fp)
	{
		char *colon = strchr( temp, WIREGL_PATH_SEPARATOR[0] );
		if (!colon)
			colon = temp+strlen(temp);
		strncpy(fname,temp,colon-temp);
		fname[colon-temp] = '\0';
		strcat(fname,"/");
		strcat(fname,filename);
		if ((fp = fopen(fname, "r")) != NULL)
		{
			fclose(fp);
			found_it = 1;
			break;
		}
		temp = colon;
		if (*temp)
			temp++;
		if (!*temp)
			break;
	}
	if (!found_it)
		wireGLSimpleError( "Can't find %s in WIREGL_PATH", filename );

	num_pipe_servers = 0;

	wireGLParseFile( fname, wiregl_parse, NULL );
	InitiateServerConnections();
}
