/*
** glx.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include <GL/glx.h>

#include "wiregl/include/wiregl_client.h"
#include "wiregl/include/wiregl_util.h"

#if defined( IRIX ) || defined( IRIX64 )
#define SYSTEM_GL "/usr/lib32/libGL.so"
#elif defined( Linux )
#define SYSTEM_GL "/usr/lib/libGL.so"
#endif

extern Display *wiregl_glx_display;
extern GLXDrawable wiregl_glx_drawable;

static struct {
    int init;
    XVisualInfo* (*ChooseVisual)(Display *, int, int *);
    void         (*CopyContext)(Display *, GLXContext, GLXContext, GLuint);
    GLXContext   (*CreateContext)(Display *, XVisualInfo *, GLXContext, Bool);
    GLXPixmap    (*CreateGLXPixmap)(Display *, XVisualInfo *, Pixmap);
    void         (*DestroyContext)(Display *, GLXContext);
    void         (*DestroyGLXPixmap)(Display *, GLXPixmap);
    int          (*GetConfig)(Display *, XVisualInfo *vis, int, int *);
    GLXContext   (*GetCurrentContext)(void);
    GLXDrawable  (*GetCurrentDrawable)(void);
    Bool         (*IsDirect)(Display *, GLXContext);
    Bool         (*MakeCurrent)(Display *, GLXDrawable, GLXContext);
    Bool         (*QueryExtension)(Display *, int *, int *);
    Bool         (*QueryVersion)(Display *, int *, int *);
    void         (*SwapBuffers)(Display *, GLXDrawable);
    void         (*UseXFont)(Font, int, int, int);
    void         (*WaitGL)(void);
    void         (*WaitX)(void);
    const char * (*QueryExtensionsString)(Display *, int);
    const char * (*GetClientString)(Display *, int);
    const char * (*QueryServerString)(Display *, int, int);
} __sysGLX;

#define SYSGLX(cmd)      ((__sysGLX.init || __glx_LoadSystemGL( )), __sysGLX.##cmd)

static void *
MapDllEntry( WireGLDSO *handle, const char *name )
{
    void *fptr = wireGLDSOGet( handle, name );

    if ( fptr == NULL ) {
			wireGLSimpleError( "Failed to get the symbol %s", name );
    }

    return fptr;
}


void
__glx_LoadSystemGL( void )
{
    __sysGLX.init = 1;

    WireGLDSO * libGLsys = wireGLDSOOpen( SYSTEM_GL );
    if ( libGLsys == NULL ) {
			wireGLSimpleError( "Couldn't open %s", SYSTEM_GL );
    }

#define MAP(a,b)        __sysGLX.##a = MapDllEntry( libGLsys, b )

    MAP( ChooseVisual,          "glXChooseVisual"          );
    //MAP( CopyContext,           "glXCopyContext"           );
    //MAP( CreateContext,         "glXCreateContext"         );
    //MAP( CreateGLXPixmap,       "glXCreateGLXPixmap"       );
    //MAP( DestroyContext,        "glXDestroyContext"        );
    //MAP( DestroyGLXPixmap,      "glXDestroyGLXPixmap"      );
    //MAP( GetConfig,             "glXGetConfig"             );
    //MAP( GetCurrentContext,     "glXGetCurrentContext"     );
    //MAP( GetCurrentDrawable,    "glXGetCurrentDrawable"    );
    //MAP( IsDirect,              "glXIsDirect"              );
    //MAP( MakeCurrent,           "glXMakeCurrent"           );
    //MAP( QueryExtension,        "glXQueryExtension"        );
    //MAP( QueryVersion,          "glXQueryVersion"          );
    //MAP( SwapBuffers,           "glXSwapBuffers"           );
    //MAP( UseXFont,              "glXUseXFont"              );
    //MAP( WaitGL,                "glXWaitGL"                );
    //MAP( WaitX,                 "glXWaitX"                 );
    //MAP( QueryExtensionsString, "glXQueryExtensionsString" );
    //MAP( GetClientString,       "glXGetClientString"       );
    //MAP( QueryServerString,     "glXQueryServerString"     );

#undef MAP
}

XVisualInfo *
glXChooseVisual( Display *dpy, int screen, int *attribList )
{
	/*
	XVisualInfo *vis = (XVisualInfo *) wireGLAlloc( sizeof( XVisualInfo ) );
	vis->bits_per_rgb = 24;
	vis->colormap_size = 256;
	vis->blue_mask = 0x0000ff;
	vis->green_mask = 0x00ff00;
	vis->red_mask = 0xff0000;
	vis->depth = 24;
	vis->screen = 0;
	vis->visualid = 0;
	vis->visual = (Visual *) wireGLAlloc( sizeof( Visual ) );
	vis->visual->ext_data = NULL;
	vis->visual->visualid = vis->visualid;
	vis->visual->red_mask = vis->red_mask;
	vis->visual->green_mask = vis->green_mask;
	vis->visual->blue_mask = vis->blue_mask;
	vis->visual->bits_per_rgb = vis->bits_per_rgb;
	vis->visual->map_entries = vis->colormap_size;
	return vis;
	*/

	if (!__sysGLX.init)
	{
		__sysGLX.init = 1;
		__glx_LoadSystemGL();
	}

	XVisualInfo *vis =  __sysGLX.ChooseVisual( dpy, screen, attribList );
	return vis;
}

void
glXCopyContext( Display *dpy, GLXContext src, GLXContext dst, GLuint mask )
{
	wireGLSimpleError( "Unsupported GLX Call: glxCopyContext" );
}

GLXContext
glXCreateContext( Display *dpy, XVisualInfo *vis, GLXContext share,
                  Bool direct )
{
	//wireGLCreateContext();
	return (GLXContext) "foo";
}

GLXPixmap
glXCreateGLXPixmap( Display *dpy, XVisualInfo *vis, Pixmap pixmap )
{
	wireGLSimpleError( "Unsupported GLX Call: glxcreateglxpixmap" );
	return (GLXPixmap) 0;
}

void
glXDestroyContext( Display *dpy, GLXContext ctx )
{
	wireGLSimpleError( "Unsupported GLX Call: glxdestroycontext" );
}

void
glXDestroyGLXPixmap( Display *dpy, GLXPixmap pix )
{
	wireGLSimpleError( "Unsupported GLX Call: glxdestroyglxpixmap" );
}

int
glXGetConfig( Display *dpy, XVisualInfo *vis, int attrib, int *value )
{
    switch ( attrib ) {

      case GLX_USE_GL:
        *value = 1;
        break;

      case GLX_BUFFER_SIZE:
        *value = 32;
        break;

      case GLX_LEVEL:
        *value = 0;
        break;

      case GLX_RGBA:
        *value = 1;
        break;

      case GLX_DOUBLEBUFFER:
        *value = 1;
        break;

      case GLX_STEREO:
        *value = 0;
        break;

      case GLX_AUX_BUFFERS:
        *value = 0;
        break;

      case GLX_RED_SIZE:
        *value = 8;
        break;

      case GLX_GREEN_SIZE:
        *value = 8;
        break;

      case GLX_BLUE_SIZE:
        *value = 8;
        break;

      case GLX_ALPHA_SIZE:
        *value = 8;
        break;

      case GLX_DEPTH_SIZE:
        *value = 8;
        break;

      case GLX_STENCIL_SIZE:
        *value = 8;
        break;

      case GLX_ACCUM_RED_SIZE:
        *value = 0;
        break;

      case GLX_ACCUM_GREEN_SIZE:
        *value = 0;
        break;

      case GLX_ACCUM_BLUE_SIZE:
        *value = 0;
        break;

      case GLX_ACCUM_ALPHA_SIZE:
        *value = 0;
        break;

      default:
        return GLX_BAD_ATTRIBUTE;
    }

    return 0;
}

GLXContext
glXGetCurrentContext( void )
{
	wireGLSimpleError( "Unsupported GLX Call: glxgetcurrentcontext" );
	return (GLXContext) 0;
}

GLXDrawable
glXGetCurrentDrawable( void )
{
	wireGLSimpleError( "Unsupported GLX Call: glxgetcurrentdrawable" );
	return (GLXDrawable) 0;
}

Bool
glXIsDirect( Display *dpy, GLXContext ctx )
{
	return 1;
}

void
__glX_WindowGetSize( Display *display, GLXDrawable drawable,
                     unsigned int *width_retval,
                     unsigned int *height_retval )
{
    Window       root;
    int          x, y;
    unsigned int width, height, border, depth;

    if ( !XGetGeometry( display, drawable, &root, &x, &y,
                        &width, &height, &border, &depth ) )
        wireGLSimpleError( "__glX_WindowGetSize( ): XGetGeometry() failed!" );

    *width_retval  = width;
    *height_retval = height;
}


Bool
glXMakeCurrent( Display *dpy, GLXDrawable drawable, GLXContext ctx )
{
	if (wiregl_glx_display != dpy)
	{
		wiregl_glx_display = dpy;
		wiregl_glx_drawable = drawable;
		wireGLCreateContext(); // moved here by Humper.
	}
	__glMakeCurrent( pipe_server_state );
  return True;
}

Bool
glXQueryExtension( Display *dpy, int *errorBase, int *eventBase )
{
	return 1; // You BET we do...
}

Bool
glXQueryVersion( Display *dpy, int *major, int *minor )
{
    *major = 1;
    *minor = 3;

    return 1;
}

void
glXSwapBuffers( Display *dpy, GLXDrawable drawable )
{
	wireGLSwapBuffers();
}

void
glXUseXFont( Font font, int first, int count, int listBase )
{
	wireGLSimpleError( "Unsupported GLX Call: glxusexfont" );
}

void
glXWaitGL( void )
{
	wireGLSimpleError( "Unsupported GLX Call: glxwaitgl" );
}

void
glXWaitX( void )
{
	wireGLSimpleError( "Unsupported GLX Call: glxwaitx" );
}

const char *
glXQueryExtensionsString( Display *dpy, int screen )
{
    static const char *retval = "";

    return retval;
}

const char *
glXGetClientString( Display *dpy, int name )
{
    const char *retval;
    const char *namestr;
    char buf[32];

    switch ( name ) {

      case GLX_VENDOR:
        namestr = "GLX_VENDOR";
        retval  = "WireGL";
        break;

      case GLX_VERSION:
        namestr = "GLX_VERSION";
        retval  = "1.0 WireGL";
        break;

      case GLX_EXTENSIONS:
        namestr = "GLX_EXTENSIONS";
        retval  = "";
        break;

      default:
        sprintf( buf, "0x%04x", name );
        namestr = buf;
        retval  = NULL;
    }

    return retval;
}

const char *
glXQueryServerString( Display *dpy, int screen, int name )
{
    const char *retval;
    const char *namestr;
    char buf[32];

    switch ( name ) {

      case GLX_VENDOR:
        namestr = "GLX_VENDOR";
        retval  = "WireGL";
        break;

      case GLX_VERSION:
        namestr = "GLX_VERSION";
        retval  = "1.0 WireGL";
        break;

      case GLX_EXTENSIONS:
        namestr = "GLX_EXTENSIONS";
        retval  = "";
        break;

      default:
        sprintf( buf, "0x%04x", name );
        namestr = buf;
        retval  = NULL;
    }

    return retval;
}
