#include "wiregl/include/wireGL.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_client.h"
#include "wiregl/include/wiregl_util.h"

#define GET_BUFFERED_COUNT_POINTER( len ) \
  data_ptr = pipe_server_data_current; \
  if (data_ptr + (len) > pipe_server_data_end || bucket_count == bucket_max)\
  { \
    wireGLFlushBuffer( pipe_server_state ); \
	data_ptr = pipe_server_data_current; \
  } \
  pipe_server_data_current += (len); \
  bucket_count++

#define UPDATE_BBOX() __asm { \
	__asm movaps xmm1, &f;\
	__asm movaps xmm2, &bounds_min; \
	__asm movaps xmm3, &bounds_max; \
	__asm minps xmm2, xmm1; \
	__asm maxps xmm3, xmm1; \
	__asm movaps &bounds_min, xmm2; \
	__asm movaps &bounds_max, xmm3; \
}

#define CREATE_2D_FLOATS() \ 
	GLvectorf f;
	f.x = (GLfloat) x; \
	f.y = (GLfloat) y

#define CREATE_3D_FLOATS() \ 
	GLvectorf f;
	f.x = (GLfloat) x; \
	f.y = (GLfloat) y; \
	f.z = (GLfloat) z

#define CREATE_4D_FLOATS() \ 
	GLvectorf f;
	f.x = (GLfloat) x; \
	f.y = (GLfloat) y; \
	f.z = (GLfloat) z; \
	f.w = (GLfloat) w; \
	f.x /= f.w; f.y /= f.w; f.z /= f.w;

#define CREATE_2D_VFLOATS() \ 
	GLvectorf f;
	f.x = (GLfloat) v[0]; \
	f.y = (GLfloat) v[1]

#define CREATE_3D_VFLOATS() \ 
	GLvectorf f;
	f.x = (GLfloat) v[0]; \
	f.y = (GLfloat) v[1]; \
	f.z = (GLfloat) v[2]

#define CREATE_4D_VFLOATS() \ 
	GLvectorf f;
	f.x = (GLfloat) v[0]; \
	f.y = (GLfloat) v[1]; \
	f.z = (GLfloat) v[2]; \
	f.w = (GLfloat) v[3]; \
	f.x /= f.w; f.y /= f.w; f.z /= f.w;

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2d ) ( GLdouble x, GLdouble y )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 16 );
	CREATE_2D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLdouble, x );
	WRITE_DATA( 8, GLdouble, y );
	WRITE_OPCODE( WIREGL_VERTEX2D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2dv ) ( const GLdouble *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 16 );
	CREATE_2D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLdouble, v[0] );
	WRITE_DATA( 8, GLdouble, v[1] );
	WRITE_OPCODE( WIREGL_VERTEX2D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2f ) ( GLfloat x, GLfloat y )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 8 );
	CREATE_2D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLfloat, x );
	WRITE_DATA( 4, GLfloat, y );
	WRITE_OPCODE( WIREGL_VERTEX2F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2fv ) ( const GLfloat *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 8 );
	CREATE_2D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLfloat, v[0] );
	WRITE_DATA( 4, GLfloat, v[1] );
	WRITE_OPCODE( WIREGL_VERTEX2F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2i ) ( GLint x, GLint y )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 8 );
	CREATE_2D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLint, x );
	WRITE_DATA( 4, GLint, y );
	WRITE_OPCODE( WIREGL_VERTEX2I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2iv ) ( const GLint *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 8 );
	CREATE_2D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLint, v[0] );
	WRITE_DATA( 4, GLint, v[1] );
	WRITE_OPCODE( WIREGL_VERTEX2I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2s ) ( GLshort x, GLshort y )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 4 );
	CREATE_2D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLshort, x );
	WRITE_DATA( 2, GLshort, y );
	WRITE_OPCODE( WIREGL_VERTEX2S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD2sv ) ( const GLshort *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 4 );
	CREATE_2D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLshort, v[0] );
	WRITE_DATA( 2, GLshort, v[1] );
	WRITE_OPCODE( WIREGL_VERTEX2S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3d ) ( GLdouble x, GLdouble y, GLdouble z )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 24 );
	CREATE_3D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLdouble, x );
	WRITE_DATA( 8, GLdouble, y );
	WRITE_DATA( 16, GLdouble, z );
	WRITE_OPCODE( WIREGL_VERTEX3D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3dv ) ( const GLdouble *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 24 );
	CREATE_3D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLdouble, v[0] );
	WRITE_DATA( 8, GLdouble, v[1] );
	WRITE_DATA( 16, GLdouble, v[2] );
	WRITE_OPCODE( WIREGL_VERTEX3D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3f ) ( GLfloat x, GLfloat y, GLfloat z )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 12 );
	CREATE_3D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLfloat, x );
	WRITE_DATA( 4, GLfloat, y );
	WRITE_DATA( 8, GLfloat, z );
	WRITE_OPCODE( WIREGL_VERTEX3F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3fv ) ( const GLfloat *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 12 );
	CREATE_3D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLfloat, v[0] );
	WRITE_DATA( 4, GLfloat, v[1] );
	WRITE_DATA( 8, GLfloat, v[2] );
	WRITE_OPCODE( WIREGL_VERTEX3F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3i ) ( GLint x, GLint y, GLint z )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 12 );
	CREATE_3D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLint, x );
	WRITE_DATA( 4, GLint, y );
	WRITE_DATA( 8, GLint, z );
	WRITE_OPCODE( WIREGL_VERTEX3I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3iv ) ( const GLint *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 12 );
	CREATE_3D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLint, v[0] );
	WRITE_DATA( 4, GLint, v[1] );
	WRITE_DATA( 8, GLint, v[2] );
	WRITE_OPCODE( WIREGL_VERTEX3I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3s ) ( GLshort x, GLshort y, GLshort z )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 6 );
	CREATE_3D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLshort, x );
	WRITE_DATA( 2, GLshort, y );
	WRITE_DATA( 4, GLshort, z );
	WRITE_OPCODE( WIREGL_VERTEX3S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD3sv ) ( const GLshort *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 6 );
	CREATE_3D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLshort, v[0] );
	WRITE_DATA( 2, GLshort, v[1] );
	WRITE_DATA( 4, GLshort, v[2] );
	WRITE_OPCODE( WIREGL_VERTEX3S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4d ) ( GLdouble x, GLdouble y, GLdouble z, GLdouble w )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 32 );
	CREATE_4D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLdouble, x );
	WRITE_DATA( 8, GLdouble, y );
	WRITE_DATA( 16, GLdouble, z );
	WRITE_DATA( 24, GLdouble, w );
	WRITE_OPCODE( WIREGL_VERTEX4D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4dv ) ( const GLdouble *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 32 );
	CREATE_4D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLdouble, v[0] );
	WRITE_DATA( 8, GLdouble, v[1] );
	WRITE_DATA( 16, GLdouble, v[2] );
	WRITE_DATA( 24, GLdouble, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4f ) ( GLfloat x, GLfloat y, GLfloat z, GLfloat w )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 16 );
	CREATE_4D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLfloat, fx );
	WRITE_DATA( 4, GLfloat, fy );
	WRITE_DATA( 8, GLfloat, fz );
	WRITE_DATA( 12, GLfloat, 1.0f );
	WRITE_OPCODE( WIREGL_VERTEX4F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4fv ) ( const GLfloat *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 16 );
	CREATE_4D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLfloat, v[0] );
	WRITE_DATA( 4, GLfloat, v[1] );
	WRITE_DATA( 8, GLfloat, v[2] );
	WRITE_DATA( 12, GLfloat, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4i ) ( GLint x, GLint y, GLint z, GLint w )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 16 );
	CREATE_4D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLint, x );
	WRITE_DATA( 4, GLint, y );
	WRITE_DATA( 8, GLint, z );
	WRITE_DATA( 12, GLint, w );
	WRITE_OPCODE( WIREGL_VERTEX4I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4iv ) ( const GLint *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 16 );
	CREATE_4D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLint, v[0] );
	WRITE_DATA( 4, GLint, v[1] );
	WRITE_DATA( 8, GLint, v[2] );
	WRITE_DATA( 12, GLint, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4s ) ( GLshort x, GLshort y, GLshort z, GLshort w )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 8 );
	CREATE_4D_FLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLshort, x );
	WRITE_DATA( 2, GLshort, y );
	WRITE_DATA( 4, GLshort, z );
	WRITE_DATA( 6, GLshort, w );
	WRITE_OPCODE( WIREGL_VERTEX4S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB_SIMD4sv ) ( const GLshort *v )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 8 );
	CREATE_4D_VFLOATS();
	UPDATE_BBOX();
	WRITE_DATA( 0, GLshort, v[0] );
	WRITE_DATA( 2, GLshort, v[1] );
	WRITE_DATA( 4, GLshort, v[2] );
	WRITE_DATA( 6, GLshort, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4S_OPCODE );
}

