/*
** viewport.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include "wiregl/include/wireGL.h"
#include "wiregl/include/wiregl_client.h"
#include "wiregl/include/wiregl_protocol.h"

#ifdef WINDOWS
#include <windows.h>

HDC client_hdc;
HWND client_hwnd;
#else
#include <X11/Xlib.h>

typedef void * GLXDrawable;

Display *wiregl_glx_display;
GLXDrawable wiregl_glx_drawable;
#endif

#include <stdio.h>

int __getWindowSize( int *width_return, int *height_return )
{
#ifdef WINDOWS
	RECT r;
	if (!client_hwnd)
	{
		client_hwnd = WindowFromDC( client_hdc );
	}
	if (!client_hwnd)
	{
		return 0;
	}
	GetClientRect( client_hwnd, &r );
	*width_return = r.right - r.left;
	*height_return = r.bottom - r.top;

#else

	Window       root;
	int          x, y;
	unsigned int width, height, border, depth;

	if (!wiregl_glx_display)
	{
		return 0;
	}

	if ( !XGetGeometry( wiregl_glx_display, (Drawable) wiregl_glx_drawable, &root, &x, &y,
				&width, &height, &border, &depth ) )
		wireGLSimpleError( "XGetGeometry failed" );

	*width_return  = width;
	*height_return = height;
#endif
	return 1;
}

int __computeNormalizedWindowCoordinates( GLint x, GLint y, GLsizei width, GLsizei height,
		float *nx, float *ny, float *nwidth, float *nheight )
{
	int have_useful_data = 0;
	int client_width = 1, client_height = 1;
	have_useful_data = __getWindowSize( &client_width, &client_height );

	*nx = ((float)x)/((float)client_width);
	*nwidth = ((float)width)/((float)client_width);

	*ny = ((float)y)/((float)client_height);
	*nheight = ((float)height)/((float)client_height);
	
	return have_useful_data;
}

static void __handleWindowCoordsCall( GLint x, GLint y, GLsizei width, GLsizei height )
{
	unsigned char *data_ptr;
/*	int have_useful_data = 0;
	float nx = 0, ny = 0, nwidth = 0, nheight = 0;

	have_useful_data = __computeNormalizedWindowCoordinates( x, y, width, height, &nx, &ny, &nwidth, &nheight );
	WRITE_DATA( 0, int, have_useful_data );
	if (have_useful_data)
	{ 
*/
	GET_BUFFERED_POINTER( 16 );
	WRITE_DATA( 0, GLint, x );
	WRITE_DATA( 4, GLint, y );
	WRITE_DATA( 8, GLint, width );
	WRITE_DATA( 12, GLint, height );
/*
	}
*/
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( Viewport  )(GLint x, GLint y, GLsizei width, GLsizei height )
{
	__handleWindowCoordsCall( x, y, width, height );
	WRITE_OPCODE( WIREGL_VIEWPORT_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( Scissor  )( GLint x, GLint y, GLsizei width, GLsizei height )
{
	__handleWindowCoordsCall( x, y, width, height );
	WRITE_OPCODE( WIREGL_SCISSOR_OPCODE );
}
