/*
** wgl.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/


#include <windows.h>
#include <stdio.h>

#include "wiregl/glcontext/glcontext.h"
#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_client.h"

BOOL  ( WINAPI *wglUseFontOutlinesA_proc ) (HDC, DWORD, DWORD, DWORD, FLOAT, FLOAT, int, LPGLYPHMETRICSFLOAT);
BOOL  ( WINAPI *wglCopyContext_proc ) (HGLRC, HGLRC, UINT);
HGLRC ( WINAPI *wglCreateLayerContext_proc ) (HDC, int);
PROC  ( WINAPI *wglGetProcAddress_proc ) (LPCSTR);
BOOL  ( WINAPI *wglUseFontBitmapsA_proc ) (HDC, DWORD, DWORD, DWORD);
BOOL  ( WINAPI *wglDescribeLayerPlane_proc ) (HDC, int, int, UINT, LPLAYERPLANEDESCRIPTOR );
int   ( WINAPI *wglSetLayerPaletteEntries_proc ) (HDC, int, int, int, CONST COLORREF *);
int   ( WINAPI *wglGetLayerPaletteEntries_proc ) (HDC, int, int, int, COLORREF *);
BOOL  ( WINAPI *wglRealizeLayerPalette_proc ) (HDC, int, BOOL);
DWORD ( WINAPI *wglSwapMultipleBuffers_proc ) (UINT, const void *);
BOOL  ( WINAPI *wglSwapLayerBuffers_proc ) (HDC, UINT);

int		( WINAPI *wglChoosePixelFormat_proc) (HDC a, CONST PIXELFORMATDESCRIPTOR *b);
BOOL	( WINAPI *wglSetPixelFormat_proc) (HDC a, int b, CONST PIXELFORMATDESCRIPTOR *c);
HGLRC	( WINAPI *wglCreateContext_proc) (HDC);
BOOL	( WINAPI *wglDeleteContext_proc)(HGLRC);
BOOL	( WINAPI *wglMakeCurrent_proc) (HDC, HGLRC);
HGLRC	( WINAPI *wglGetCurrentContext_proc) (VOID);
HDC		( WINAPI *wglGetCurrentDC_proc) (VOID);
int		( WINAPI *wglGetPixelFormat_proc) (HDC);
int		( WINAPI *wglDescribePixelFormat_proc) (HDC a, int b, UINT c, LPPIXELFORMATDESCRIPTOR  d);
BOOL	( WINAPI *wglShareLists_proc) (HGLRC a, HGLRC b);
BOOL    ( WINAPI *wglSwapBuffers_proc) (HDC a);

/* Unimplemented...
**
** wglGetDefaultProcAddress
** wglDescribeLayerPlane
** wglGetLayerPaletteEntries
** wglRealizeLayerPalette_prox
** wglSetLayerPaletteEntries_prox
** wglSwapLayerBuffers_prox

** BOOL  WINAPI wglCopyContext(HGLRC, HGLRC, UINT);
** HGLRC WINAPI wglCreateLayerContext(HDC, int);
** PROC  WINAPI wglGetProcAddress(LPCSTR);
** BOOL  WINAPI wglShareLists(HGLRC, HGLRC);
** BOOL  WINAPI wglUseFontBitmapsA(HDC, DWORD, DWORD, DWORD);
** BOOL  WINAPI wglUseFontBitmapsW(HDC, DWORD, DWORD, DWORD);
**
*/

HINSTANCE hDll = NULL;


BOOL WINAPI DllMain(
  HINSTANCE /* hinstDLL */,  // handle to DLL module
  DWORD /* fdwReason */,     // reason for calling function
  LPVOID /* lpvReserved  */ // reserved
  ) {
	if (!hDll) {
		/* Find the real opengl32.dll */		
		char name[256];
		GetSystemDirectory(name, 256);
		strcat (name, "\\opengl32.dll");
		if (!hDll)  hDll=LoadLibrary(name);
		if (!hDll)  wireGLSimpleError("Couldn't Load opengl32.dll");

		wglChoosePixelFormat_proc = (int (WINAPI *) (HDC a, CONST PIXELFORMATDESCRIPTOR *b))
			GetProcAddress(hDll, "wglChoosePixelFormat");
		wglSetPixelFormat_proc = (BOOL ( WINAPI *) (HDC a, int b, CONST PIXELFORMATDESCRIPTOR *c))
			GetProcAddress(hDll, "wglSetPixelFormat");
		wglCreateContext_proc = (HGLRC ( WINAPI *) (HDC))
			GetProcAddress(hDll, "wglCreateContext");
		wglDeleteContext_proc = (BOOL ( WINAPI *) (HGLRC))
			GetProcAddress(hDll, "wglDeleteContext");
		wglMakeCurrent_proc = (BOOL ( WINAPI *) (HDC, HGLRC))
			GetProcAddress(hDll, "wglMakeCurrent");
		wglGetCurrentContext_proc = (HGLRC ( WINAPI *) (VOID))
			GetProcAddress(hDll, "wglGetCurrentContext");
		wglGetCurrentDC_proc = (HDC ( WINAPI *) (VOID))
			GetProcAddress(hDll, "wglGetCurrentDC");
		wglGetPixelFormat_proc = (int ( WINAPI *) (HDC))
			GetProcAddress(hDll, "wglGetPixelFormat");
		wglDescribePixelFormat_proc = (int ( WINAPI *) (HDC, int, UINT, LPPIXELFORMATDESCRIPTOR))
			GetProcAddress(hDll, "wglDescribePixelFormat");
		wglShareLists_proc = (BOOL	( WINAPI *) (HGLRC a, HGLRC b))
			GetProcAddress(hDll, "wglShareLists");
		wglSwapBuffers_proc = (BOOL  ( WINAPI *) (HDC a))
			GetProcAddress(hDll, "wglSwapBuffers");
		wglUseFontOutlinesA_proc = (BOOL  ( WINAPI *) (HDC, DWORD, DWORD, DWORD, FLOAT, FLOAT, int, LPGLYPHMETRICSFLOAT))
			GetProcAddress(hDll, "wglUseFontOutlinesA");
		wglCopyContext_proc = (BOOL  ( WINAPI *) (HGLRC, HGLRC, UINT))
			GetProcAddress(hDll, "wglCopyContext");
		wglCreateLayerContext_proc = (HGLRC ( WINAPI *) (HDC, int))
			GetProcAddress(hDll, "wglCreateLayerContext");
		wglGetProcAddress_proc = (PROC  ( WINAPI * ) (LPCSTR))
			GetProcAddress(hDll, "wglGetProcAddress");
		wglUseFontBitmapsA_proc = (BOOL  ( WINAPI *) (HDC, DWORD, DWORD, DWORD))
			GetProcAddress(hDll, "wglUseFontBitmapsA");
		wglDescribeLayerPlane_proc = (BOOL  ( WINAPI *) (HDC, int, int, UINT, LPLAYERPLANEDESCRIPTOR ))
			GetProcAddress(hDll, "wglDescribeLayerPlane");
		wglSetLayerPaletteEntries_proc = (int   ( WINAPI *) (HDC, int, int, int, CONST COLORREF *))
			GetProcAddress(hDll, "wglSetLayerPaletteEntries");
		wglGetLayerPaletteEntries_proc = (int   ( WINAPI *) (HDC, int, int, int, COLORREF *))
			GetProcAddress(hDll, "wglGetLayerPaletteEntries");
		wglRealizeLayerPalette_proc = (BOOL  ( WINAPI *) (HDC, int, BOOL))
			GetProcAddress(hDll, "wglRealizeLayerPalette");
		wglSwapMultipleBuffers_proc = (DWORD ( WINAPI *) (UINT, CONST void *))
			GetProcAddress(hDll, "wglSwapMultipleBuffers");
		wglSwapLayerBuffers_proc = (BOOL  ( WINAPI *) (HDC, UINT))
			GetProcAddress(hDll, "wglSwapLayerBuffers");
	}
	return TRUE;
}


int	 WINAPI wglChoosePixelFormat_prox (HDC a, CONST PIXELFORMATDESCRIPTOR *b) {
	return wglChoosePixelFormat_proc(a, b);
}

BOOL  WINAPI wglSetPixelFormat_prox (HDC a, int b, CONST PIXELFORMATDESCRIPTOR *c) {
	return wglSetPixelFormat_proc (a, b, c);
}

BOOL  WINAPI wglDeleteContext_prox (HGLRC a) {
	__glDestroyContext( pipe_server_state );
	return wglDeleteContext_proc (a);
}

BOOL  WINAPI wglMakeCurrent_prox (HDC /* a */, HGLRC /* b */) {
	__glMakeCurrent( pipe_server_state );
	/* return wglMakeCurrent_proc (a, b); */
	return TRUE;
}

HGLRC  WINAPI wglGetCurrentContext_prox (VOID) {
	return wglGetCurrentContext_proc ();
}

HDC  WINAPI wglGetCurrentDC_prox (VOID){
	return wglGetCurrentDC_proc ();
}

int	  WINAPI wglGetPixelFormat_prox (HDC a) {
	return wglGetPixelFormat_proc(a);
}
 
int  WINAPI wglDescribePixelFormat_prox (HDC a, int b, UINT c, LPPIXELFORMATDESCRIPTOR  d) {
	return wglDescribePixelFormat_proc(a, b, c, d);
}

BOOL  WINAPI wglShareLists_prox (HGLRC a, HGLRC b) {
	return wglShareLists_proc(a, b);
}
HGLRC  WINAPI wglCreateContext_prox (HDC a) {
	HGLRC hglrc;

	client_hdc = a;
	client_hwnd = WindowFromDC( client_hdc );
	hglrc = wglCreateContext_proc (a);

	wireGLCreateContext();

	return hglrc;
}

BOOL  WINAPI wglSwapBuffers_prox(HDC /* a */) {
	wireGLSwapBuffers();
	return TRUE;
}

BOOL  WINAPI wglCopyContext_prox(HGLRC a, HGLRC b, UINT c)
{
	return wglCopyContext_proc( a,b,c );
}

HGLRC WINAPI wglCreateLayerContext_prox(HDC a, int b)
{
	return  wglCreateLayerContext_proc( a,b );
}

PROC  WINAPI wglGetProcAddress_prox(LPCSTR a)
{
	wireGLSimpleError("Error wglGetProcAddress not implemented.");
	return wglGetProcAddress_proc( a );
}

BOOL  WINAPI wglUseFontBitmapsA_prox(HDC a, DWORD b, DWORD c, DWORD d)
{
	return wglUseFontBitmapsA_proc( a,b,c,d );
}

BOOL  WINAPI wglDescribeLayerPlane_prox(HDC a, int b, int c, UINT d,LPLAYERPLANEDESCRIPTOR e)
{
	return wglDescribeLayerPlane_proc( a,b,c,d,e );
}

int   WINAPI wglSetLayerPaletteEntries_prox(HDC a, int b, int c, int d, CONST COLORREF * e)
{
	return wglSetLayerPaletteEntries_proc( a,b,c,d,e );
}

int   WINAPI wglGetLayerPaletteEntries_prox(HDC a, int b, int c, int d, COLORREF * e)
{
	return wglGetLayerPaletteEntries_proc( a,b,c,d,e );
}

BOOL  WINAPI wglRealizeLayerPalette_prox(HDC a, int b, BOOL c)
{
	return wglRealizeLayerPalette_proc( a,b,c );
}

DWORD WINAPI wglSwapMultipleBuffers_prox(UINT a, CONST void * b)
{
	return wglSwapMultipleBuffers_proc( a,b );
}

BOOL  WINAPI wglUseFontOutlinesA_prox(HDC a, DWORD b, DWORD c, DWORD d, FLOAT e, FLOAT f, int g, LPGLYPHMETRICSFLOAT h)
{
	return wglUseFontOutlinesA_proc( a,b,c,d,e,f,g,h );
}

BOOL  WINAPI wglSwapLayerBuffers_prox(HDC a, UINT b)
{
	return wglSwapLayerBuffers_proc( a,b );
}
