/*
** arrays.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifdef WINDOWS
#include <windows.h>
#endif
#include <GL/gl.h>
#include "wiregl/include/wireGL.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/include/wiregl_util.h"
#include "wiregl/glcontext/glcontext.h"

void __decodeVertexPointer( void ) {
	GLint size = READ_DATA(0, GLint);
	GLenum type = READ_DATA(4, GLenum);
	__glstate_VertexPointer (size, type, 0, NULL);
	glVertexPointer(size, type, 0, wiregl_pipe.state->client.v.cache);
	INCR_DATA_PTR( 8 );
}

void __decodeColorPointer( void ) {
	GLint size = READ_DATA(0, GLint);
	GLenum type = READ_DATA(4, GLenum);
	__glstate_ColorPointer(size, type,	0, NULL);
	glColorPointer(size, type, 0, wiregl_pipe.state->client.c.cache);
	INCR_DATA_PTR( 8 );
}

void __decodeIndexPointer( void ) {
	GLenum type = READ_DATA(0, GLenum);
	__glstate_IndexPointer(type, 0, NULL);
	glIndexPointer(type, 0, wiregl_pipe.state->client.i.cache);
	INCR_DATA_PTR( 4 );
}

void __decodeNormalPointer( void ) {
	GLenum type = READ_DATA(0, GLenum);
	__glstate_NormalPointer(type, 0, NULL);
	glNormalPointer(type, 0, wiregl_pipe.state->client.n.cache);
	INCR_DATA_PTR( 4 );
}

void __decodeTexCoordPointer( void ) {
	GLint size = READ_DATA(0, GLint);
	GLenum type = READ_DATA(4, GLenum);
	__glstate_TexCoordPointer(size, type, 0, NULL);
	glTexCoordPointer(size, type, 0, wiregl_pipe.state->client.t.cache);
	INCR_DATA_PTR( 8 );
}

void __decodeEdgeFlagPointer( void ) {
	glEdgeFlagPointer(0, wiregl_pipe.state->client.e.cache);
}

void __decodeEnableClientState( void )
{
	GLenum array = READ_DATA( 0, GLenum );
	glEnableClientState( array  );
	__glstate_EnableClientState (array);
	INCR_DATA_PTR( 4 );
}

void __decodeDisableClientState( void )
{
	GLenum array = READ_DATA( 0, GLenum );
	glDisableClientState( array  );
	__glstate_DisableClientState (array);
	INCR_DATA_PTR( 4 );
}

void __decodeDrawArrays ( void  ) {

	glDrawArrays(
		READ_DATA(0, GLenum), 
		READ_DATA(4, GLint),
		READ_DATA(8, GLsizei));
	INCR_DATA_PTR( 12 );
}

void __decodeArrayElement ( void  ) {
	glArrayElement( READ_DATA(0, GLint) );
	INCR_DATA_PTR( 4 );
}

void __decodeUpdateClientPointer ( void ) {
	int array = READ_DATA(4, int);
	int index_size = READ_DATA(8, int);
	int data_size = READ_DATA(12, int);
	int *index = DATA_POINTER(16, int);
	unsigned char *data = DATA_POINTER(16+index_size*sizeof(int), unsigned char);

	__glclient_UpdateClientPointer(array, index_size, index, data_size, data);

	INCR_VAR_PTR();
}


