/*
** main.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifdef WINDOWS
#include <windows.h>
#endif
#include <GL/gl.h>


#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_common.h"

#include "wiregl/glcontext/glcontext.h"

WireGLPipe wiregl_pipe;

static void __parseTrace( char *file )
{
	wiregl_pipe.fname = file;
}

static void __parsePort( char *port )
{
	sscanf( port, "%d", &(wiregl_pipe.port) );
}

static struct flags {
	char *flag;
	void (*fn)(char *);
} parse_structs[] = {
	{"-p", __parsePort},
	{"-t", __parseTrace},
	{NULL, NULL}
};

static void __setDefaults( void )
{
	wiregl_pipe.port = WIREGL_PORT;
	wiregl_pipe.window_x = 0;
	wiregl_pipe.window_y = 0;
	wiregl_pipe.window_width = 1024;
	wiregl_pipe.window_height = 768;
	wiregl_pipe.fname = NULL;
	wiregl_pipe.swap_barrier = 0;
	wiregl_pipe.barrier_conn = NULL;
}

static void __parseArgs( int argc, char **argv )
{
	int i,j;

	__setDefaults();

	for (i = 0 ; i < argc ; i++)
	{
		for (j = 0 ; parse_structs[j].flag ; j++)
		{
			if (!strcmp( argv[i], parse_structs[j].flag ))
			{
				parse_structs[j].fn( argv[i+1] );
				i++;
				break;
			}
		}
	}
}

int main( int argc, char *argv[] )
{
	(void) wireGLInitInstrumentation( TO_SCREEN );
	(void) wireGLInitOS();
	(void) wireGLInitWarnings();

	__parseArgs( argc, argv );

	GLconfig *c = __glconfig_CreateDefaultConfig();
	
	__glInit(c);

	wiregl_pipe.state = __glCreateContext();
	__glMakeCurrent( wiregl_pipe.state );

	wireGLWaitForConnection(); // will be the fname, or NULL
	wireGLCreateWindow();
	wireGLRenderRemoteStream();
	return 0;
}
