/*
** server_net.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifdef WINDOWS
#include <winsock.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#endif

#include <stdio.h>
#include <errno.h>

#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_common.h"

void wireGLReadManyPackets( WireGLConnection *conn, int *num_opcodes, unsigned  char **opcodes, char **data )
{
	*num_opcodes = conn->wireGLReadMessage( conn, conn->receive_buf_one );
	if (*num_opcodes < 0)
	{
		wireGLSimpleError( "Client disconnected" );
	}
	if (conn->wireGLReadMessage( conn, conn->receive_buf_two ) < 0)
	{
		wireGLSimpleError( "Client disconnected" );
	}

	*opcodes = (unsigned char *) conn->receive_buf_one;
	*data = (char *) conn->receive_buf_two;
}

#define MAX_MAP_NAME_LEN 4096

void wireGLWaitForConnection( )
{
	WireGLClientConnection connection;
	WireGLServer *server = wireGLBecomeServer( wiregl_pipe.port );

	if (wiregl_pipe.fname != NULL)
	{
		wiregl_pipe.conn = wireGLAcceptFileClient( wiregl_pipe.fname );
	}
	else
	{
		char map_fname[MAX_MAP_NAME_LEN];
		strcpy( map_fname, "" );

		wiregl_pipe.conn = wireGLAcceptClient( server, &connection );

		wiregl_pipe.window_x = connection.window_x;
		wiregl_pipe.window_y = connection.window_y;
		wiregl_pipe.window_width = connection.window_width;
		wiregl_pipe.window_height = connection.window_height;
		wiregl_pipe.feather_top_size = connection.feather_top_size;
		wiregl_pipe.feather_top[0] = connection.feather_top[0];
		wiregl_pipe.feather_top[1] = connection.feather_top[1];
		wiregl_pipe.feather_top[2] = connection.feather_top[2];
		wiregl_pipe.feather_top[3] = connection.feather_top[3];
		wiregl_pipe.feather_bottom_size = connection.feather_bottom_size;
		wiregl_pipe.feather_bottom[0] = connection.feather_bottom[0];
		wiregl_pipe.feather_bottom[1] = connection.feather_bottom[1];
		wiregl_pipe.feather_bottom[2] = connection.feather_bottom[2];
		wiregl_pipe.feather_bottom[3] = connection.feather_bottom[3];
		wiregl_pipe.feather_left_size = connection.feather_left_size;
		wiregl_pipe.feather_left[0] = connection.feather_left[0];
		wiregl_pipe.feather_left[1] = connection.feather_left[1];
		wiregl_pipe.feather_left[2] = connection.feather_left[2];
		wiregl_pipe.feather_left[3] = connection.feather_left[3];
		wiregl_pipe.feather_right_size = connection.feather_right_size;
		wiregl_pipe.feather_right[0] = connection.feather_right[0];
		wiregl_pipe.feather_right[1] = connection.feather_right[1];
		wiregl_pipe.feather_right[2] = connection.feather_right[2];
		wiregl_pipe.feather_right[3] = connection.feather_right[3];
	}
	wireGLFinishAcceptClient( wiregl_pipe.conn );
	wiregl_pipe.state->viewport.x = 0;
	wiregl_pipe.state->viewport.y = 0;
	wiregl_pipe.state->viewport.w = wiregl_pipe.window_width;
	wiregl_pipe.state->viewport.h = wiregl_pipe.window_height;

}
