/*
** stream.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifdef WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <GL/gl.h>
#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_util.h"
#include "feather.h"

//#define TIME_SEND

extern char *glNames[];

char *unpacker_data_ptr;

void wireGLRenderRemoteStream( void )
{
	int num_opcodes;
	char *data = NULL;
	unsigned char *opcodes = NULL;

#ifdef TIME_SEND
	LARGE_INTEGER feqtime;
	float feq;
	QueryPerformanceFrequency(&feqtime);
	feq = (float) feqtime.QuadPart;
#endif

	for (;;)
	{
		int i;

#ifdef TIME_SEND
		LARGE_INTEGER pretime, posttime, difftime;
		float readtime;
		QueryPerformanceCounter(&pretime);
#endif

		wireGLReadManyPackets( wiregl_pipe.conn, &num_opcodes, &opcodes, &data );

#ifdef TIME_SEND
		QueryPerformanceCounter(&posttime);
		difftime.QuadPart = posttime.QuadPart;
		difftime.QuadPart -= pretime.QuadPart;
		readtime = (float) difftime.QuadPart;
		printf("wireGLReadManyPackets: %f\n", readtime/feq);
#endif

		unpacker_data_ptr = data;

#if WINDOWS
		MSG msg;
		while (PeekMessage(&msg, NULL, 0, 0xffffffff, PM_REMOVE)) {
			TranslateMessage (&msg);
			DispatchMessage (&msg);
		}
#endif

		for (i = 0 ; i < num_opcodes ; i++)
		{
#if 0
			if (strcmp(glNames[opcodes[i]], "Vertex3f") &&
				strcmp(glNames[opcodes[i]], "Normal3f"))
				printf ("%d/%d:%d: %s\n", i+1, num_opcodes, opcodes[i], glNames[opcodes[i]] );
#endif
#if 0
			if (!strcmp(glNames[opcodes[i]], "SwapBuffers"))
				printf("------------------------\n");
#endif
			wiregl_decode_functions[opcodes[i]]();
#if 0
			if (__glGetCurrentContext()->current.beginend == GL_FALSE) {
				GLenum err = glGetError();
				while (err != GL_NO_ERROR) {
					static const char errorstring[][32] = {"GL_INVALID_ENUM",
						"GL_INVALID_VALUE",
						"GL_INVALID_OPERATION",
						"GL_STACK_OVERFLOW",
						"GL_STACK_UNDERFLOW",
						"GL_OUT_OF_MEMORY"};

					printf ("%s: GLerror: %s\n", glNames[opcodes[i]], errorstring[err-0x500]);
					err = glGetError();
				}
			}
#endif
		}

	}
}
