/*
** window.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#define NO_INCLUDE_GLCONTEXT_STUFF

#ifdef WINDOWS
#include <windows.h>
#endif
#include <GL/gl.h>

#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_util.h"

#include "wiregl/include/wireGL.h" // so we don't conflict with other stuff
#ifndef WINDOWS
#if defined( IRIX ) || defined( IRIX64 ) || defined( Linux )
#include "glxtokens.h"
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xmu/StdCmap.h>
#include <X11/Xatom.h>
#include <sys/time.h>

typedef void *GLXContext;
typedef XID GLXDrawable;

#ifdef __cplusplus
#define C_EXTERN "C"
#else
#define C_EXTERN
#endif

extern C_EXTERN Bool glXQueryExtension (Display *dpy, int *errorBase, int *eventBase);
extern C_EXTERN XVisualInfo * glXChooseVisual (Display *dpy, int screen, int *attribList);
extern C_EXTERN GLXContext glXCreateContext (Display *dpy, XVisualInfo *vis, GLXContext shareList, Bool direct);
extern C_EXTERN Bool glXMakeCurrent (Display *dpy, GLXDrawable drawable, GLXContext ctx);
extern C_EXTERN void glXSwapBuffers (Display *dpy, GLXDrawable drawable);

#else
#include <windows.h>
#endif

#include <stdlib.h>
#include <stdio.h>


#ifndef WINDOWS

static int configuration[] = { GLX_DOUBLEBUFFER, GLX_RGBA, GLX_DEPTH_SIZE, 8, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1, GLX_BLUE_SIZE, 1, None };


static char clear_bits[] = {
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

static Colormap wireGLGetShareableColormap( Display *dpy, XVisualInfo *vi )
{
	Status status;
	XStandardColormap *standardCmaps;
	Colormap cmap;
	int i, numCmaps;

	if (vi->c_class != TrueColor)
		wireGLError ("No support for non-TrueColor visuals.");
	status = XmuLookupStandardColormap( dpy, vi->screen, vi->visualid, vi->depth,
			XA_RGB_DEFAULT_MAP, False, True );
	if ( status == 1 )
	{
		status = XGetRGBColormaps( dpy, RootWindow( dpy, vi->screen), 
				&standardCmaps, &numCmaps, 
				XA_RGB_DEFAULT_MAP );
		if ( status == 1 )
		{
			for (i = 0 ; i < numCmaps ; i++)
			{
				if (standardCmaps[i].visualid == vi->visualid)
				{
					cmap = standardCmaps[i].colormap;
					XFree( standardCmaps);
					return cmap;
				}
			}
		}
	}
	cmap = XCreateColormap( dpy, RootWindow(dpy, vi->screen), 
			vi->visual, AllocNone);
	return cmap;
}

/* ARGSUSED */
	static int
WaitForMapNotify(Display *display, XEvent *event, char *arg)
{
	return(event->type == MapNotify && event->xmap.window == (Window)arg);
}

#else
static LONG WINAPI MainWndProc( HWND, UINT, WPARAM, LPARAM );
BOOL bSetupPixelFormat( HDC );
volatile BOOL gDone = 0;
	
#endif

void wireGLCreateWindow( )
{
#ifndef WINDOWS
	Colormap cmap;
	XSetWindowAttributes swa;
	XSizeHints hints = {0};
	XEvent event;
	char p_display[1024];
	char *wireGL_display;
	XTextProperty text_prop;
	XClassHint *class_hints = NULL;
	Pixmap pixmap;
	Cursor cursor;
	XColor color;
	char *name;
#else
	HINSTANCE hinstance;
	WNDCLASS wc;
#endif

#ifndef WINDOWS
	wireGL_display = getenv("WIREGL_DISPLAY");
	if (wireGL_display)
		sprintf( p_display, "%s:0", wireGL_display );
	else
	{
		char *foo = getenv( "DISPLAY" );
		if (foo)
		{
			strcpy( p_display, foo );
		}
		else
		{
			strcpy( p_display, ":0" );
		}
	}

	wireGLWarning( WIREGL_WARN_DEBUG, "Computed the display as %s", p_display );
	wiregl_pipe.dpy = XOpenDisplay( p_display );
	wireGLWarning( WIREGL_WARN_DEBUG, "Opened the display: %p", wiregl_pipe.dpy );

	if (!wiregl_pipe.dpy)
		wireGLError( "Couldn't open display %s", p_display );
	if (!glXQueryExtension( wiregl_pipe.dpy, NULL, NULL ))
		wireGLError( "Display %s doesn't support GLX", p_display );

	wireGLWarning (WIREGL_WARN_DEBUG, "Looks like we have GLX" );

	XSetScreenSaver( wiregl_pipe.dpy, 0, 0, PreferBlanking, AllowExposures );
	wireGLWarning( WIREGL_WARN_DEBUG, "Just turned off the screensaver" );
	wiregl_pipe.vi = glXChooseVisual( wiregl_pipe.dpy, DefaultScreen( wiregl_pipe.dpy ), configuration );
	if (!wiregl_pipe.vi)
		wireGLError( "Display %s doesn't have the necessary visual.", p_display );
	wireGLWarning( WIREGL_WARN_DEBUG, "Chose a visual" );
	cmap = wireGLGetShareableColormap( wiregl_pipe.dpy, wiregl_pipe.vi );
	wireGLWarning( WIREGL_WARN_DEBUG, "Chose a colormap" );
	wiregl_pipe.cx = glXCreateContext( wiregl_pipe.dpy, wiregl_pipe.vi, NULL, GL_TRUE );
	wireGLWarning( WIREGL_WARN_DEBUG, "Created a context" );

  /* Make a clear cursor to get rid of the monitor cursor */

	if (wiregl_pipe.cx == NULL)
		wireGLError( "Couldn't create rendering context" ); 
	swa.colormap = cmap;
	swa.border_pixel = 0;
	swa.event_mask = ExposureMask | StructureNotifyMask;

	wiregl_pipe.win = XCreateWindow( wiregl_pipe.dpy, 
			RootWindow( wiregl_pipe.dpy, wiregl_pipe.vi->screen ),
			wiregl_pipe.window_x, wiregl_pipe.window_y,
			wiregl_pipe.window_width , wiregl_pipe.window_height,
			0, wiregl_pipe.vi->depth, InputOutput, wiregl_pipe.vi->visual,
			CWBorderPixel | CWColormap 
			| CWEventMask,
			&swa );

  pixmap = XCreatePixmapFromBitmapData(wiregl_pipe.dpy, wiregl_pipe.win, 
				       clear_bits, 16, 16, 1, 0, 1);
  cursor = XCreatePixmapCursor(wiregl_pipe.dpy, pixmap, pixmap, &color, &color, 8, 8);
  XDefineCursor(wiregl_pipe.dpy, wiregl_pipe.win, cursor);

	wireGLWarning( WIREGL_WARN_DEBUG, "Created the window" );
	hints.x = wiregl_pipe.window_x;
	hints.y = wiregl_pipe.window_y;
	hints.width = wiregl_pipe.window_width;
	hints.height = wiregl_pipe.window_height;
	hints.min_width = hints.width;
	hints.min_height = hints.height;
	hints.max_width = hints.width;
	hints.max_height = hints.height;
	hints.flags = USPosition | USSize | PMinSize | PMaxSize;
	XSetStandardProperties( wiregl_pipe.dpy, wiregl_pipe.win, "a wireGL window", "no icon for you",
			None, NULL, 0, &hints );
#if 1
	/* New item!  This is needed so that the sgimouse server can find
	 * the wireGLserver window. 
	 */
	name = "WireGL Window";
	XStringListToTextProperty(&name, 1, &text_prop);
	XSetWMName(wiregl_pipe.dpy, wiregl_pipe.win, &text_prop);
#endif
	class_hints = XAllocClassHint();
	class_hints->res_name = strdup("foo");
	class_hints->res_class = strdup("WireGL");
	XSetClassHint( wiregl_pipe.dpy, wiregl_pipe.win, class_hints );
	free( class_hints->res_name );
	free( class_hints->res_class );
	XFree( class_hints );

	wireGLWarning( WIREGL_WARN_DEBUG, "About to make current to the context" );

	XMapWindow( wiregl_pipe.dpy, wiregl_pipe.win );
	XIfEvent( wiregl_pipe.dpy, &event, WaitForMapNotify, (char *) wiregl_pipe.win );
	if (!glXMakeCurrent( wiregl_pipe.dpy, wiregl_pipe.win, wiregl_pipe.cx ))
		wireGLError(" Error making current" );
	wireGLWarning( WIREGL_WARN_DEBUG, "Made current to the context" );
#else
	hinstance = GetModuleHandle(NULL);
	
	if (!GetClassInfo(hinstance, "PC WireGL!", &wc)) {
		
		wc.style = CS_OWNDC;
		wc.lpfnWndProc = (WNDPROC)MainWndProc;
		wc.cbClsExtra = 0;
		wc.cbWndExtra = 0;
		wc.hInstance = hinstance;
		wc.hIcon = LoadIcon( NULL, IDI_APPLICATION );
		wc.hCursor = LoadCursor( NULL, IDC_ARROW );
		wc.hbrBackground = NULL;
		wc.lpszMenuName = NULL;
		wc.lpszClassName = "PC WireGL!";
		
		if (!RegisterClass( &wc ))
		{
			wireGLError( "Couldn't register window class -- you're not trying to do multi-pipe stuff on windows, are you?" );
		}
	}
	
	wiregl_pipe.hWnd = CreateWindow( "PC WireGL!", "PC WireGL!", WS_POPUP | WS_SYSMENU, 
										wiregl_pipe.window_x, wiregl_pipe.window_y,
										wiregl_pipe.window_width, wiregl_pipe.window_height,
										NULL, NULL, hinstance, &wiregl_pipe );

	if (!wiregl_pipe.hWnd)
	{
		wireGLError( "Create Window failed" );
	}

	ShowWindow( wiregl_pipe.hWnd, SW_SHOWNORMAL );
	SetForegroundWindow(wiregl_pipe.hWnd);
	if (!getenv ("WIREGL_NO_TOP"))
		SetWindowPos(wiregl_pipe.hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	ShowCursor(FALSE);
	//SetCursorPos(wiregl_pipe.window_width, wiregl_pipe.window_height);
	wiregl_pipe.device_context = GetDC( wiregl_pipe.hWnd );
	if (!bSetupPixelFormat( wiregl_pipe.device_context ) )
		wireGLError( "Couldn't set up the device context" );
	wiregl_pipe.hRC = wglCreateContext( wiregl_pipe.device_context );
	wglMakeCurrent( wiregl_pipe.device_context, wiregl_pipe.hRC );

#endif
	glClearColor( 0, 0, 0, 1 );
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
#ifndef WINDOWS
	glXSwapBuffers( wiregl_pipe.dpy, wiregl_pipe.win );
#else
	SwapBuffers( wiregl_pipe.device_context );
#endif

	/*
	glViewport( -wiregl_pipe.mural_x, -wiregl_pipe.mural_y, 
			wiregl_pipe.mural_width, wiregl_pipe.mural_height );
	*/
	glFinish();
}

void wireGLPipeKillWindow() {
#ifdef WINDOWS
	wglMakeCurrent(GetDC(wiregl_pipe.hWnd), NULL);
	wglDeleteContext(wiregl_pipe.hRC);
	DestroyWindow(wiregl_pipe.hWnd);
	wiregl_pipe.hWnd = NULL;
#endif
}

#ifdef WINDOWS
LONG WINAPI
MainWndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	int w,h;
	switch (uMsg) {
	case WM_SIZE:
		w = LOWORD( lParam );
		h = HIWORD( lParam );
	
		glViewport(0, 0, w, h);
/*
	glViewport( -wiregl_pipe.mural_x, -wiregl_pipe.mural_y, 
			wiregl_pipe.mural_width, wiregl_pipe.mural_height );
	glScissor( -wiregl_pipe.mural_x, -wiregl_pipe.mural_y, 
			wiregl_pipe.mural_width, wiregl_pipe.mural_height );
*/
	}
	return DefWindowProc( hWnd, uMsg, wParam, lParam );
}

BOOL bSetupPixelFormat(HDC hdc)
{
    PIXELFORMATDESCRIPTOR *ppfd; 
    PIXELFORMATDESCRIPTOR pfd = { 
	sizeof(PIXELFORMATDESCRIPTOR),  //  size of this pfd 
	1,                              // version number 
	PFD_DRAW_TO_WINDOW |            // support window 
	PFD_SUPPORT_OPENGL |            // support OpenGL 
	PFD_DOUBLEBUFFER,               // double buffered 
	PFD_TYPE_RGBA,                  // RGBA type 
	24,                             // 24-bit color depth 
	0, 0, 0, 0, 0, 0,               // color bits ignored 
	0,                              // no alpha buffer 
	0,                              // shift bit ignored 
	0,                              // no accumulation buffer 
	0, 0, 0, 0,                     // accum bits ignored 
	32,                             // 32-bit z-buffer	 
	0,                              // no stencil buffer 
	0,                              // no auxiliary buffer 
	PFD_MAIN_PLANE,                 // main layer 
	0,                              // reserved 
	0, 0, 0                         // layer masks ignored 
    }; 
    int pixelformat;

	//  Commented out by Ian.
    //pfd.cColorBits = GetDeviceCaps(hdc,BITSPIXEL);
	ppfd = &pfd;
    pixelformat = ChoosePixelFormat(hdc, ppfd); 
    if ( (pixelformat = ChoosePixelFormat(hdc, ppfd)) == 0 ) {
        MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK); 
        return FALSE; 
    }
    if (SetPixelFormat(hdc, pixelformat, ppfd) == FALSE) {
        MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK); 
        return FALSE; 
    }
    return TRUE; 
}
#endif


