/*
** os.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include "__utilsource.h"

#include <stdlib.h>
#if !defined(WINDOWS)
#include <unistd.h>
#include <limits.h>
#else
#include <windows.h>
#include <winbase.h>
#endif
#include <sys/stat.h> /* WHY is mkdir in here? */
#include <string.h>
#include "wiregl/include/wiregl_util.h"

#if defined( IRIX ) || defined( IRIX64 )
usptr_t *shared_mem = NULL;
static char *temp_filename;
#endif

UTIL_DECL void wireGLInitShmem( char *filename )
{
#if defined(IRIX) || defined(IRIX64)
	if (!(shared_mem = usinit( filename ) ) )
		wireGLError( "Couldn't create the shared memory on IRIX" );
#else
	(void) filename;
	wireGLWarning( WIREGL_WARN_CRITICAL, "Init Shmem on windows no worky." );
#endif
}


UTIL_DECL char *wireGLInitOS(void)
{
#if defined( IRIX ) || defined( IRIX64 )
	char *size = getenv( "WIREGL_SHMEM_SIZE" );
	int shmem_size = 1048576;

	if (size)
	{
		sscanf( size, "%d", &shmem_size );
		wireGLWarning( WIREGL_WARN_CRITICAL, "Setting the shared memory size "
				"to %d", shmem_size );
	}

	mkdir ( "/tmp/wireGL2", 0777 );
	system( "rm -f /tmp/wireGL2/*" );
	temp_filename = tempnam( "/tmp/wireGL2", "wireGL2" );
	usconfig( CONF_INITUSERS, 500 ); 
	usconfig( CONF_INITSIZE, shmem_size );
	usconfig( CONF_ATTACHADDR, 0x50000000 );
	wireGLInitShmem( temp_filename );
	usconfig( CONF_ATTACHADDR, ~0 );
	return temp_filename;
#elif defined(WINDOWS)
	WORD wVersionRequested = MAKEWORD(2, 0);
	WSADATA wsaData;
	int err;

	err = WSAStartup(wVersionRequested, &wsaData);
	if (err != 0)
		wireGLError("Couldn't initialize sockets on WINDOWS");
#endif
	return "no shared memory"; /* This function has to return *something* */
}
UTIL_DECL void *wireGLAllocShared( size_t bytes )
{
	void *ret;
#if defined(IRIX) || defined(IRIX64)
	ret = usmalloc( bytes, shared_mem );
#else
	wireGLWarning( WIREGL_WARN_CRITICAL, "You can't allocate shared memory on WINDOWS!" );
	return NULL; /* STUPID FUCKING WINDOWS COMPILER WON'T SHUT UP */
#endif
	if (!ret)
		wireGLError( "Couldn't allocate shared %d bytes", bytes );
	return ret;
}

UTIL_DECL void wireGLFreeShared( void *ptr )
{
#if defined(IRIX) || defined(IRIX64)
	usfree( ptr, shared_mem );
#else
	(void) ptr;
	wireGLWarning( WIREGL_WARN_CRITICAL, "You can't free shared memory on WINDOWS!" );
#endif
}

UTIL_DECL void wireGLSleep( unsigned int msecs )
{
#if defined ( IRIX ) || defined( IRIX64 )
	sginap( (long) msecs * (CLK_TCK / 1000) );
#elif defined( WINDOWS )
	Sleep( msecs );
#elif defined( Linux )
	usleep( msecs * 1000 );
#else
#error UNKNOWN SLEEPING ARCHITECTURE
#endif
}


UTIL_DECL void wireGLCreateThread( WireGLThreadFunc entry, void *arg )
{
#if defined( IRIX ) || defined( IRIX64 )
	if ( sproc( entry, PR_SADDR | PR_SFDS, arg ) < 0 )
		wireGLError( "sproc() failure on IRIX" );
#endif
#if defined(WINDOWS)
	u_long id;

	if (CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) entry, (LPVOID) arg, 0, &id) == NULL)
		wireGLError ("CreateThread failure on WINDOWS");
#endif
#if defined(Linux)
	pthread_t newThreadID;
	if(pthread_create(&newThreadID, NULL, (void *(*)(void *))entry,
				arg) !=0)
	{
		wireGLError ("pthread_create failure on Linux");
	}
#endif
}
