/*
** parse.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include "__utilsource.h"

#include "wiregl/include/wiregl_util.h"

#include <stdio.h>
#include <ctype.h>

UTIL_DECL char *wireGLGetLine( FILE *fp, char *buf )
{
	for (;;)
	{
		char *ass = fgets(buf,1024,fp);
		if (feof(fp) || !ass)
			return NULL;
		if (!buf[0] || buf[0] == '\n' || buf[0] == '#')
			continue;
		break;
	}
	if (buf[strlen(buf) - 1] == '\n')
		buf[strlen(buf) - 1] = '\0';
	return buf;
}

static char *EatWhiteSpace( char *buf )
{
	while (*buf &&isspace( *buf ))
		buf++;
	return buf;
}

UTIL_DECL void wireGLParseFile( char *fname, WireGLParseStruct *parsers, void *arg )
{
	FILE *fp = fopen( fname, "r" );
	if (!fp)
		wireGLSimpleError( "Can't open %s", fname );

	char buf[1024];
	memset( buf, 0, sizeof(buf) );

	while ( wireGLGetLine( fp, buf ) != NULL )
	{
		WireGLParseStruct *ps;
		for (ps = parsers ; ps->str != NULL ; ps++)
		{
			if (!strncmp( buf, ps->str, strlen( ps->str ) ))
			{
				char *rest_of_line = &(buf[0]) + strlen(ps->str);
				rest_of_line = EatWhiteSpace( rest_of_line );
				ps->parser( rest_of_line, arg );
				break;
			}
		}
		if (!ps->str)
		{
			wireGLSimpleError( "Parse Error: %s", buf );
		}
	}
}
