#include "__utilsource.h"

#include <stdio.h>
#include <errno.h>

#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_instrument.h"

#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

#define SHMEM_BUF_SIZE (16*1024*1024)

#define DEBUG(x)
//#define DEBUG( x ) x

#define CONSUMER_HEAD_OFFSET( conn ) (*(conn->p_consumer_head_offset))
#define CONSUMER_TAIL_OFFSET( conn ) (*(conn->p_consumer_tail_offset))
#define PRODUCER_HEAD_OFFSET( conn ) (*(conn->p_producer_head_offset))
#define PRODUCER_TAIL_OFFSET( conn ) (*(conn->p_producer_tail_offset))

int __computeBytesAvailable( WireGLConnection *conn, int needed, int *bytes_1, int *bytes_2 )
{
	int bytes_available;
	int head = CONSUMER_HEAD_OFFSET( conn );
	int tail = CONSUMER_TAIL_OFFSET( conn );

	if (tail >= head)
	{
		bytes_available = MIN( tail-head, needed );
		*bytes_1 = *bytes_2 = 0;
	}
	else
	{
		if (SHMEM_BUF_SIZE - head >= needed)
		{
			bytes_available = needed;
			*bytes_1 = *bytes_2 = 0;
		}
		else
		{
			DEBUG( printf ("WRAP!\n") );
			*bytes_1 = SHMEM_BUF_SIZE - head;
			*bytes_2 = MIN( needed - *bytes_1, tail );
			bytes_available = *bytes_1 + *bytes_2;
		}
	}
	return bytes_available;
}
	
int __computeSpaceAvailable( WireGLConnection *conn, int needed, int *bytes_1, int *bytes_2 )
{
	int space_available;
	int head = PRODUCER_HEAD_OFFSET( conn );
	int tail = PRODUCER_TAIL_OFFSET( conn );

	DEBUG( printf ("head: %d\ttail: %d\n", head, tail ) );
	if (SHMEM_BUF_SIZE - tail >= needed)
	{
		if (head > tail)
		{
			space_available = MIN( needed, head-tail-1); // gotta leave one byte
			DEBUG( printf( "1: %d\n", space_available ) );
		}
		else
		{
			if (head == 0 && (SHMEM_BUF_SIZE - tail == needed))
			{
				space_available = needed-1;
				DEBUG( printf( "2: %d\n", space_available ) );
			}
			else
			{
				space_available = needed;
				DEBUG( printf( "3: %d\n", space_available ) );
			}
		}
		*bytes_1 = *bytes_2 = 0;
	}
	else
	{
		if (head > tail )
		{
			space_available = head-tail-1;
			DEBUG( printf( "4: %d\n", space_available ) );
			*bytes_1 = *bytes_2 = 0;
		}
		else
		{
			if (head == 0)
			{
				space_available = SHMEM_BUF_SIZE - tail - 1;
				*bytes_1 = 0;
				*bytes_2 = 0;
				DEBUG( printf( "5: %d\n", space_available ) );
			}
			else
			{
				*bytes_1 = SHMEM_BUF_SIZE - tail;
				*bytes_2 = MIN( head - 1, needed - *bytes_1 );
				DEBUG( printf( "6: %d %d\n", *bytes_1, *bytes_2 ) );
			}
			space_available = *bytes_1 + *bytes_2;
		}
	}
	return space_available;
}
	
static int wireGLShmemReadExact( WireGLConnection *conn, int needed, void *_buf )
{
	char *copy_ptr = (char *) _buf;
	wireGLAcquireSpinLock( conn->consumer_lock );
	while (needed > 0)
	{
		int bytes_available;
		int bytes_1, bytes_2;
		bytes_available = __computeBytesAvailable( conn, needed, &bytes_1, &bytes_2 );
		DEBUG( printf ("bytes_available = %d, bytes_1 = %d, bytes_2 = %d\n", bytes_available, bytes_1, bytes_2 )) ;
		while (bytes_available == 0)
		{
			DEBUG( printf ("There were NO bytes available, so I'm waiting.\n") );
			wireGLWaitCondVar( conn->bytesProduced, conn->consumer_lock );
			bytes_available = __computeBytesAvailable( conn, needed, &bytes_1, &bytes_2 );
			DEBUG( printf ("bytes_available = %d, bytes_1 = %d, bytes_2 = %d\n", bytes_available, bytes_1, bytes_2 )) ;
		}
		DEBUG( printf( "\n" ) );
		wireGLReleaseSpinLock( conn->consumer_lock );
		if (!bytes_1)
		{
			if (bytes_2)
			{
				wireGLSimpleError( "Bytes1 was zero, but bytes2 wasn't." );
			}
			memcpy( copy_ptr, conn->consumer_start + CONSUMER_HEAD_OFFSET( conn ), bytes_available );
			DEBUG( printf ( "First word: %d\n", *((int *) copy_ptr) ) );
		}
		else
		{
			memcpy( copy_ptr, conn->consumer_start + CONSUMER_HEAD_OFFSET( conn ), bytes_1 );
			memcpy( copy_ptr + bytes_1, conn->consumer_start, bytes_2 );
		}
		needed -= bytes_available;
		copy_ptr += bytes_available;
		wireGLAcquireSpinLock( conn->consumer_lock );
		DEBUG( printf ("Bytes produced: %d\n", conn->shmem->bytes_produced ) );
		DEBUG( printf ("Bytes consumed: %d\n", conn->shmem->bytes_consumed ) );
		conn->shmem->bytes_consumed += bytes_available;
		if (!bytes_1)
		{
			CONSUMER_HEAD_OFFSET( conn ) += bytes_available;
			if (CONSUMER_HEAD_OFFSET( conn ) == SHMEM_BUF_SIZE)
				CONSUMER_HEAD_OFFSET( conn ) = 0;
			if (CONSUMER_HEAD_OFFSET( conn ) >= SHMEM_BUF_SIZE)
				wireGLSimpleError( "Head has gone crazy" );
		}
		else
		{
			CONSUMER_HEAD_OFFSET( conn ) = bytes_2;
		}
		wireGLSignalCondVar( conn->bytesConsumed );
	}
	wireGLReleaseSpinLock( conn->consumer_lock );
	return 1;
}

UTIL_DECL int wireGLShmemReadMessage( WireGLConnection *conn, void *buf )
{
	int len;
	wireGLShmemReadExact( conn, sizeof( len ), &len );
	wireGLShmemReadExact( conn, len, buf );
	return len;
}

UTIL_DECL void wireGLShmemSendMessage( WireGLConnection *conn, void *buf, int bytes )
{
	char *src_buf = (char *) buf;

	wireGLAcquireSpinLock( conn->producer_lock );
	while( bytes > 0 )
	{
		int space_available;
		int bytes_1, bytes_2;
		space_available = __computeSpaceAvailable( conn, bytes, &bytes_1, &bytes_2 );
		DEBUG( printf ( "Space available = %d, bytes_1 = %d, bytes_2 = %d\n", space_available, bytes_1, bytes_2 ) );
		while (space_available == 0)
		{
			DEBUG( printf ("There was no space, so I'm waiting.\n") );
			wireGLWaitCondVar( conn->bytesConsumed, conn->producer_lock );
			space_available = __computeSpaceAvailable( conn, bytes, &bytes_1, &bytes_2 );
			DEBUG( printf ("Space available = %d, bytes_1 = %d, bytes_2 = %d\n", space_available, bytes_1, bytes_2 ) );
		}
		DEBUG( printf( "\n" ) );
		wireGLReleaseSpinLock( conn->producer_lock );
		if (!bytes_1)
		{
			if (bytes_2)
			{
				wireGLSimpleError( "Bytes1 was zero, but bytes2 wasn't." );
			}
			DEBUG( printf( "First word: %d\n", *((int *) src_buf) ) );
			memcpy( conn->producer_start + PRODUCER_TAIL_OFFSET( conn ), src_buf, space_available );
		}
		else
		{
			memcpy( conn->producer_start + PRODUCER_TAIL_OFFSET( conn ), src_buf, bytes_1 );
			memcpy( conn->producer_start, src_buf + bytes_1, bytes_2 );
		}
		bytes -= space_available;
		src_buf += space_available;
		wireGLAcquireSpinLock( conn->producer_lock );
		conn->shmem->bytes_produced += space_available;
		if (!bytes_1)
		{
			PRODUCER_TAIL_OFFSET( conn ) += space_available;
			if (PRODUCER_TAIL_OFFSET( conn ) == SHMEM_BUF_SIZE)
				PRODUCER_TAIL_OFFSET( conn ) = 0;
			if (PRODUCER_TAIL_OFFSET( conn ) >= SHMEM_BUF_SIZE)
				wireGLSimpleError("Tail has gone crazy" );
		}
		else
		{
			PRODUCER_TAIL_OFFSET( conn ) = bytes_2;
		}
		wireGLSignalCondVar( conn->bytesProduced );
	}
	wireGLReleaseSpinLock( conn->producer_lock );
}

#define FIX_WINDOWS_SHMEM_NAME( name ) \
	{ char *temp ; for (temp = (name); *temp ; temp++) if (*temp == '\\' || *temp == ':') *temp = '_'; }

UTIL_DECL void wireGLMakeShmem( WireGLConnection *conn )
{
#ifdef WINDOWS
	HANDLE hMapFile;
	LPVOID lpMapAddress;
	WireGLSharedMemoryData *server_data;
	char name[8096];

	conn->type = SHARED_MEM;
	wireGLCloseSocket( conn->tcp_socket );

	hMapFile = OpenFileMapping( FILE_MAP_ALL_ACCESS, FALSE, conn->map_filename );
	if (!hMapFile)
		wireGLSimpleError( "Couldn't map file %s", conn->map_filename );

	lpMapAddress = MapViewOfFile( hMapFile, FILE_MAP_ALL_ACCESS, 0, 0, sizeof( WireGLSharedMemoryData ) + 2*SHMEM_BUF_SIZE );
	if (!lpMapAddress)
		wireGLSimpleError( "I opened the file mapping, but couldn't map a view of it (%s)", conn->map_filename );

	server_data = (WireGLSharedMemoryData *) lpMapAddress;

	conn->shmem = server_data;

	conn->producer_start = (char *) (server_data + 1);
	conn->producer_end = conn->producer_start + SHMEM_BUF_SIZE;
	conn->consumer_start = conn->producer_end;
	conn->consumer_end = conn->consumer_start + SHMEM_BUF_SIZE;

	conn->p_consumer_head_offset = &(server_data->producer_head_offset);
	conn->p_consumer_tail_offset = &(server_data->producer_tail_offset);
	conn->p_producer_head_offset = &(server_data->consumer_head_offset);
	conn->p_producer_tail_offset = &(server_data->consumer_tail_offset);

	conn->producer_lock = &(server_data->consumer_mutex);
	conn->consumer_lock = &(server_data->producer_mutex);

	sprintf( name, "%sthreadSema", conn->map_filename );
	wireGLInitSemaphore( &(conn->threadSema), 0, name );
	conn->bytesProduced = (WireGLCondVar *) wireGLAlloc( sizeof( *(conn->bytesProduced) ) );
	conn->bytesConsumed = (WireGLCondVar *) wireGLAlloc( sizeof( *(conn->bytesProduced) ) );
	wireGLInitCondVar( conn->bytesProduced, &(conn->threadSema), &(server_data->waiting_flag) );
	wireGLInitCondVar( conn->bytesConsumed, &(conn->threadSema), &(server_data->waiting_flag) );

	conn->wireGLReadMessage = wireGLShmemReadMessage;
	conn->wireGLSendMessage = wireGLShmemSendMessage;
#else
	wireGLSimpleError( "Shared memory doesn't work except on Windows" );
#endif
}

UTIL_DECL void wireGLSetUpShmem( WireGLConnection *conn, char *map_filename )
{
#ifdef WINDOWS
	HANDLE hFile;
	HANDLE hMapFile;
	LPVOID lpMapAddress;
	char name[8096];

	hFile = CreateFile( map_filename, GENERIC_READ | GENERIC_WRITE, 
			FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, 0, NULL );
	if (!hFile)
		wireGLSimpleError( "Couldn't create %s", map_filename );

	FIX_WINDOWS_SHMEM_NAME( map_filename );

	hMapFile = CreateFileMapping( hFile, NULL, PAGE_READWRITE, 0, sizeof( WireGLSharedMemoryData ) + 2*SHMEM_BUF_SIZE, map_filename );
	if (!hMapFile)
		wireGLSimpleError( "Couldn't create file mapping!" );

	lpMapAddress = MapViewOfFile( hMapFile, FILE_MAP_ALL_ACCESS, 0, 0, sizeof( WireGLSharedMemoryData ) + 2*SHMEM_BUF_SIZE );
	if (!lpMapAddress)
		wireGLSimpleError( "Couldn't map a view of the file!" );

	conn->shmem = (WireGLSharedMemoryData *) lpMapAddress;
	conn->consumer_start = (char *) (conn->shmem + 1);
	conn->consumer_end = conn->consumer_start + SHMEM_BUF_SIZE;
	conn->producer_start = conn->consumer_end;
	conn->producer_end = conn->producer_start + SHMEM_BUF_SIZE;

	conn->shmem->bytes_produced = 0;
	conn->shmem->bytes_consumed = 0;

	conn->shmem->consumer_head_offset = 0;
	conn->shmem->consumer_tail_offset = 0;
	conn->shmem->producer_head_offset = 0;
	conn->shmem->producer_tail_offset = 0;

	conn->p_consumer_head_offset = &(conn->shmem->consumer_head_offset);
	conn->p_consumer_tail_offset = &(conn->shmem->consumer_tail_offset);
	conn->p_producer_head_offset = &(conn->shmem->producer_head_offset);
	conn->p_producer_tail_offset = &(conn->shmem->producer_tail_offset);

	wireGLInitSpinLock( &(conn->shmem->consumer_mutex) );
	wireGLInitSpinLock( &(conn->shmem->producer_mutex) );
	conn->consumer_lock = &(conn->shmem->consumer_mutex);
	conn->producer_lock = &(conn->shmem->producer_mutex);

	conn->shmem->waiting_flag = 0;

	sprintf( name, "%sthreadSema", map_filename );
	wireGLInitSemaphore( &(conn->threadSema), 0, name );
	conn->bytesProduced = (WireGLCondVar *) wireGLAlloc( sizeof( *(conn->bytesProduced) ) );
	conn->bytesConsumed = (WireGLCondVar *) wireGLAlloc( sizeof( *(conn->bytesProduced) ) );
	wireGLInitCondVar( conn->bytesProduced, &(conn->threadSema), &(conn->shmem->waiting_flag) );
	wireGLInitCondVar( conn->bytesConsumed, &(conn->threadSema), &(conn->shmem->waiting_flag) );
#else
	wireGLSimpleError( "Shared memory doesn't work except on Windows" );
#endif
}

UTIL_DECL void wireGLStartShmem( WireGLConnection *conn )
{
	wireGLCloseSocket( conn->tcp_socket );
	conn->type = SHARED_MEM;
	conn->wireGLReadMessage = wireGLShmemReadMessage;
	conn->wireGLSendMessage = wireGLShmemSendMessage;
}
