/*
** tcpip.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include "__utilsource.h"

#ifdef WINDOWS
#include <winsock.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#endif

#include <stdio.h>
#include <errno.h>

#include "wiregl/include/wiregl_util.h"

UTIL_DECL void wireGLCloseSocket( int sock )
{
#ifdef WINDOWS
	closesocket( sock );
#else
	close( sock );
#endif
}

#if 0
LARGE_INTEGER starttime;
int startinit=0;
#endif

static int wireGLTCPIPReadExact( WireGLConnection *conn, int needed, void *_buf )
{
	char *buf = (char *) _buf;
	int sock = conn->tcp_socket;
	int num_bytes;
	while (needed)
	{

#if 0
	LARGE_INTEGER feqtime;
	float feq;
	LARGE_INTEGER pretime, posttime, difftime;
	float readtime;
	if (!startinit) {
		QueryPerformanceCounter(&starttime);
		startinit = 1;
	}	
	QueryPerformanceFrequency(&feqtime);
	feq = (float) feqtime.QuadPart;
	QueryPerformanceCounter(&pretime);

#endif

	num_bytes = recv( sock, buf, needed, 0 );

#if 0
	QueryPerformanceCounter(&posttime);
	difftime.QuadPart = posttime.QuadPart;
	difftime.QuadPart -= pretime.QuadPart;
	readtime = (float) difftime.QuadPart;
	printf("%f: recv %d/%d: %f\n", (float) (pretime.QuadPart - starttime.QuadPart), num_bytes, needed, readtime/feq);
#endif


#if defined(WINDOWS)
		/* nasty hack because of the damned windows infrastructure */
		errno = WSAGetLastError();
		if (errno == WSAECONNRESET) {
			return( 0 );
		}
		errno = errno - WSABASEERR;
#endif
		if (num_bytes < 0 && errno != EINTR)
		{
#ifndef WINDOWS
			if (errno == ECONNRESET)
				return( -1 );
#endif
			return( 0 );
		}
		else if (num_bytes == 0) {

			wireGLCloseSocket( sock );
			return( -1 );
		}
		else
		{
			buf += num_bytes;
			needed -= num_bytes;
		}
	}
	return 1;
}

UTIL_DECL int wireGLTCPIPReadMessage( WireGLConnection *conn, void *buf )
{
	int len;
	if (wireGLTCPIPReadExact( conn, sizeof( len ), &len ) <= 0) return -1;
	if (wireGLTCPIPReadExact( conn, len, buf ) <= 0) return -1;
	return len;
}

UTIL_DECL void wireGLTCPIPSendMessage( WireGLConnection *conn, void *buf, int bytes )
{
#if 0
	LARGE_INTEGER feqtime;
	float feq;
	LARGE_INTEGER pretime, posttime, difftime;
	float readtime;
	if (!startinit) {
		QueryPerformanceCounter(&starttime);
		startinit = 1;
	}	
	QueryPerformanceFrequency(&feqtime);
	feq = (float) feqtime.QuadPart;
	QueryPerformanceCounter(&pretime);
#endif
	send( conn->tcp_socket, (char *) &bytes, sizeof( bytes ), 0 );

#if 0
	QueryPerformanceCounter(&posttime);
	difftime.QuadPart = posttime.QuadPart;
	difftime.QuadPart -= pretime.QuadPart;
	readtime = (float) difftime.QuadPart;
	printf("%f: send header %d: %f\n", (float) (pretime.QuadPart - starttime.QuadPart), sizeof( bytes ), readtime/feq);
	QueryPerformanceCounter(&pretime);
#endif

	send( conn->tcp_socket, (char *) buf, bytes, 0 );
#if 0
	QueryPerformanceCounter(&posttime);
	difftime.QuadPart = posttime.QuadPart;
	difftime.QuadPart -= pretime.QuadPart;
	readtime = (float) difftime.QuadPart;
	printf("%f: send data %d: %f\n", (float) (pretime.QuadPart - starttime.QuadPart), bytes, readtime/feq);
#endif
}
